<?php
/*
* 2018 - VEGA Informatique
*
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
* versions in the future. 
*
*  @author VEGA Informatique <contact@vega-info.fr>
*  @copyright  2018 VEGA Informatique
*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
*/

if (!defined('_PS_VERSION_'))
  exit;

require_once("classes/PolarisLinkChecks.php");	
require_once("classes/PolarisLinkSite.php");  
require_once("classes/PolarisLinkStockManager.php");  


/*
	Exemple d'utilisation du regroupement de modèles :
	A insérer dans product.tpl :
	
	{************************************************************ CHOIX COLORIS ************************************************************}
	{hook h='actionPolarisLinkLoadRelatedProducts'}
	{if count($relatedProducts) > 1}			
	<style>
		.related-products-selector {
		}
		
		.related-products-selector li {
			float: left;
			font-size: 8pt;
			margin-right: 4px;
		}
		
		.related-products-selector .active {
			background: darkorange;
		}
		
		.related-products-selector .active a {
			border: 1px solid darkorange;
		}
		
		.related-products-selector .active img {
			opacity: 0.7;
		}
		
		.related-products-selector a {
			display: block;
			border: 1px solid #ccc;
			border-radius: 24px;
			width: 48px;
			height: 48px;
			line-height: 24px;
			overflow: hidden;
			text-align: center;
			background: white;
			text-decoration: none;
		}
		
		.related-products-selector a:hover,
		.related-products-selector a:focus
		{
			border: 1px solid darkorange;
			text-decoration: none; 
			outline:none;
		}

	</style>
	<p style="font-weight: bold;">Existe en&nbsp;: </p>

	<!-- version 1.6 -->
	<ul class="related-products-selector">
	{foreach from=$relatedProducts item=r_product}
		<li><a {if $r_product->id == $product->id}class="active"{/if} href="{$r_product->getLink()}" title="{$r_product->name|capitalize|escape:'html':'UTF-8'}">
			{assign var="id_cover" value=$r_product->getCoverWs()}
			<img itemprop="image" src="{if $id_cover == ""}/img/p/fr-default-selection.jpg{else}{$link->getImageLink($r_product->link_rewrite, $id_cover, 'selection')|escape:'html':'UTF-8'}{/if}" title="{$r_product->name|capitalize|escape:'html':'UTF-8'}" alt="{$r_product->name|capitalize|escape:'html':'UTF-8'}" />
		</a></li>
	{/foreach}
	</ul>
	<div style="clear: both; margin-bottom: 2ex;"></div>
	{/if}

	<!-- version 1.7 -->
	<ul class="related-products-selector">
	{foreach from=$relatedProducts item=r_product}				
		<li>					
			<a {if $r_product.id_product == $product.id_product}class="active"{/if} href="{$r_product.url}" title="{$r_product.name}">						
				{if empty($r_product.cover)}
				<img src="/img/p/fr-default-selection.jpg" alt="Pas de photo" title="Pas de photo" />
				{else}
				<img src="{$r_product.cover.bySize.selection.url}" alt="{$r_product.cover.legend}" title="{$r_product.cover.legend}" />	
				{/if}
			</a>
		</li>
	{/foreach}
	</ul>
	<div style="clear: both; margin-bottom: 2ex;"></div>
	{/if}
	{************************************************************ CHOIX COLORIS ************************************************************}	
*/

class PolarisLink extends Module
{	
	public $Config = null;

	// Cache pour les produits connexes
	private $RelatedProductsInfoCache = array();

	/*
		Numéro de processus
	*/
	private static $__PROCESS_ID = 0;
	
	public function __construct()
	{
		$this->name = 'polarislink';
		$this->tab = 'synchronisation';
		$this->version = self::GetVersion();
		$this->bootstrap = true;
		$this->author = 'VEGA Informatique';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6.1.24', 'max' => _PS_VERSION_);
		$this->dependencies = array();

		parent::__construct();
 
		$this->displayName = 'PolarisLink';
		$this->description = $this->l('Permet de synchroniser votre Prestashop et votre logiciel Polaris.');
		$this->confirmUninstall = $this->l('Etes-vous sûr de vouloir désinstaller ce module ?');		
		
		if (Module::isInstalled("polarislink"))
		{
			// Chargement de la configuration
			$this->LoadConfig();
		}
		
		if (isset($_REQUEST["remount_hook"]) || Configuration::get("POLARISLINK_HOOKS_VERSION") != $this->version)
		{
			$this->registerHooks();
			Configuration::updateValue("POLARISLINK_HOOKS_VERSION", $this->version);
		}
	}

	public static function GetVersion()
	{
		return "4.08.1";
	}

	public static function GetFullVersion()
	{
		return self::GetVersion();
	}
	
	/**
	 * Conversion de sévérité
	 */
	private static function ConvertSeverityToPS($severity)
	{
		switch ($severity)
		{
			case LOG_EMERG:
			case LOG_ALERT:
			case LOG_CRIT:
				return 4;
			case LOG_ERR:
				return 3;
			case LOG_WARNING:
				return 2;
			case LOG_NOTICE:
			case LOG_INFO:
				return 1;
			case LOG_DEBUG:
				return 1;
			default:
				return 1;
		}
	}

	/**
	 * Journalisation générale
	 */
	public static function Log($severity, $msg, $linkID, $args = null, $entity = null, $isSystem = false)
	{
		if ($entity)
		{
			if (empty(self::$__PROCESS_ID))
				self::$__PROCESS_ID = uniqid();
			$msg = "PolarisLink (#".$linkID."/".self::$__PROCESS_ID.") : ".$msg;
		}

		if ($args)
			$msg = vsprintf($msg, $args);
		
		PrestaShopLogger::addLog($msg, 
			self::ConvertSeverityToPS($severity), 
			1, 
			$entity ? get_class($entity) : "PolarisLink", 
			$entity ? $entity->id : $linkID, 
			true,	// Ok pour les doublons
			$isSystem ? 0 : null	// Non attaché à un employé en particulier !
		);
	}
	
	/**
	 * Chargement de la configuration
	 */
	public function LoadConfig($check = false)
	{
		$this->Config = PolarisLinkSite::loadConfig();
		if (empty($this->Config) || count($this->Config) == 0)
		{
			$cfg = new PolarisLinkSite();
			$cfg->OPT_PAYMENT_MODULE = true;
			$cfg->save();
			
			$this->Config = array($cfg);
			$check = true;
		}
		
		if ($check)
		{
			foreach ($this->Config as $cfg)
			{
				if ($cfg->checkConfig())
					$cfg->save();
			}
		}
	}	
	
	/*
		Petite fonction bien pratique appelée pour se mettre à jour lors d'une installation.
		Cela évite de faire le boulot en double.
		Reprends tous les fichiers de mises à jour progressifs et les passes à la queue-leu-leu.
	*/
	public function upgrade_to_current()
	{
		$path = $this->local_path;
		
		$files = array();
		/* on cherche tous les upgrades */
		foreach (scandir($path."upgrade") as $file)
		{
			if (substr($file, 0, 8) == "Upgrade-")
			{
				$withoutExt = substr($file, 0, strlen($file) - 4);
				$withoutExt = substr($withoutExt, strlen("Upgrade-"));
				
				$files[$withoutExt] = $file;
			}
		}
		
		/* on va les passer dans l'ordre, version par version */
		if (!uksort($files, "version_compare"))
			$this->_errors[] = Tools::displayError("impossible de trier les versions");
		
		foreach ($files as $version => $file)
		{
			/* pour chacun d'entre eux, on passe le fichier d'upgrade */
			include ($path."upgrade/$file");

			$upgrade_func = "upgrade_module_".str_replace(".", "_", $version);			
			if (!function_exists($upgrade_func))
			{
				$this->_errors[] = Tools::displayError("impossible de trouver les méthodes d'installation de base en version $version, abandon !");
				return false;
			}
			
			if (!$upgrade_func($this))
			{
				$this->_errors[] = Tools::displayError("impossible d'installer les données de la version $version, abandon !");
				return false;
			}
		}

		$this->registerHooks();
		
		return true;
	}

	public function getHooks()
	{
		$hooks = array(
			'dashboardZoneOne',
			'displayHeader',
			'actionPolarisLinkLoadRelatedProducts',
			'actionDispatcher',
			'actionFrontControllerSetVariables',
			'displayProductPriceBlock',
			'actionPolarisLinkProductSelector',
			'actionGetAdminOrderButtons',			
			//'actionAdminOrdersListingFieldsModifier',
			//'actionAdminOrdersListingResultsModifier',
			'actionObjectCustomerAddBefore',
			'actionObjectCustomerUpdateBefore',
			'displayAdminProductsMainStepLeftColumnBottom',
			'actionObjectCategoryUpdateAfter',
			'actionObjectManufacturerUpdateAfter',
			);

		return $hooks;
	}

	public function isProductHandled($id_product)
	{
		// Chargement du produit
		foreach (ProductSupplier::getSupplierCollection($id_product) as $supplier)
		{
			foreach ($this->Config as $cfg)
			{
				if ($supplier->id_supplier == $cfg->SupplierID)
					return $cfg;
			}
		}
		return false;
	}

	public function hookDisplayAdminProductsMainStepLeftColumnBottom($params)
	{
		if (version_compare(_PS_VERSION_, 1.7) < 0)
			return null;
		$id_product = isset($params["id_product"]) ? $params["id_product"] : null;
		if (empty($id_product))
			return null;

		$product = new Product($id_product);
		$code_modele = (explode(' - ', $product->reference))[0];

		$cfg = $this->isProductHandled($id_product);
		if (!$cfg || !$cfg->OPT_ALLOW_I_CATALOG)
			return null;
	
		$polarisColor = "#e4a12d";
		$polarisBgColor = "#fff7e9";
		$style = "";
		$js = "";

		$js .= "$('#form_step1_names').append('<span style=\"font-size: 80%; font-style: italic; color: $polarisColor;\">(Les champs de cette couleur sont contrôlés par votre Backoffice Polaris et vous ne pouvez pas les modifier directement sous Prestashop)</span>');\n";
		$js .= "$('#product_reference_field').append('Code modèle Polaris : <span style=\"color: ".$polarisColor."\">".$code_modele."</span>');\n";		

		$fields_to_disabled = [
			"#form_step1_type_product",
			"#product_reference_field input", 
			"#form_step1_price_shortcut",
			"#form_step1_price_ttc_shortcut",
			"#step2_id_tax_rules_group_rendered", "#step2_id_tax_rules_group_rendered + .select2", "#step2_id_tax_rules_group_rendered + .select2 .select2-selection",
			"#form_step2_id_tax_rules_group", "#form_step2_id_tax_rules_group + .select2", "#form_step2_id_tax_rules_group + .select2 .select2-selection",
			"#form_step1_id_manufacturer", "#form_step1_id_manufacturer + .select2", "#form_step1_id_manufacturer + .select2 .select2-selection",
			"#form_step3_attributes-tokenfield", 
			"#create-combinations", 
			"#form_step2_price", "#form_step2_price_ttc", "#form_step2_unit_price",  
			"#form_step4_width", "#form_step4_height", "#form_step4_depth", "#form_step4_weight", "#form_step6_ean13", 
			"#accordion_combinations .attribute-price input", "#accordion_combinations .attribute-quantity input", 
			"#product_combination_bulk_quantity", "#product_combination_bulk_cost_price", "#product_combination_bulk_impact_on_weight", "#product_combination_bulk_impact_on_price_ti", "#product_combination_bulk_impact_on_price_te", 
			"#product_combination_bulk_reference", 
		];

		$fields_to_hide = [
			"#attributes-generator", "#attributes-list", ".combinations-list .delete", "#show_variations_selector", 
		];		

		// Nom
		if ($cfg->OPT_ALLOW_I_NAME)
			$fields_to_disabled[] = "#form_step1_name_1";

		// URL rewrite
		if ($cfg->OPT_ALLOW_I_PRODUCT_LINK)
			$fields_to_disabled[] = "#form_step5_link_rewrite_1";

		// Description
		if ($cfg->OPT_ALLOW_I_DESCRIPTIONS)
			$fields_to_disabled[] = "#form_step1_description";

		// Meta
		if ($cfg->OPT_ALLOW_I_MDESCRIPTIONS)
			$fields_to_disabled[] = "#form_step5_meta_description_1";

		// PA ?
		if ($cfg->OPT_ALLOW_I_SALE_PRICES)
			$fields_to_disabled[] = "#form_step2_wholesale_price";

		// Catégorie principale ?
		if ($cfg->AutoCategory)
			$style .= "#categories input[type=\"radio\"] { display: none; }\n#categories .main-category { display: none; }";

		// TODO : gérer peut-être les caractéristiques et les catégories dans le futur ?
		// En attendant, on le précise simplement !
		$js .= "$('#categories .categories-tree').prepend('<div class=\"polaris\">⚠ N\'oubliez pas que certaines catégories sont positionnées par Polaris !</div>');";

		$featuresAvailable = [];
		foreach (Feature::getFeatures((int) $this->context->language->id, false) as $feature)
			$featuresAvailable[(int) $feature["id_feature"]] = "<em>".strtolower($feature["name"])."</em>";

		$features = [];
		if (!empty($cfg->FeatureForAttr1) && isset($featuresAvailable[$cfg->FeatureForAttr1])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttr1];
		if (!empty($cfg->FeatureForAttr2) && isset($featuresAvailable[$cfg->FeatureForAttr2])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttr2];
		if (!empty($cfg->FeatureForAttr3) && isset($featuresAvailable[$cfg->FeatureForAttr3])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttr3];
		if (!empty($cfg->FeatureForAttr4) && isset($featuresAvailable[$cfg->FeatureForAttr4])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttr4];
		if (!empty($cfg->FeatureForAttrG1) && isset($featuresAvailable[$cfg->FeatureForAttrG1])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttrG1];
		if (!empty($cfg->FeatureForAttrG2) && isset($featuresAvailable[$cfg->FeatureForAttrG2])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttrG2];
		if (!empty($cfg->FeatureForAttrG3) && isset($featuresAvailable[$cfg->FeatureForAttrG3])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttrG3];
		if (!empty($cfg->FeatureForAttrG4) && isset($featuresAvailable[$cfg->FeatureForAttrG4])) $features[] = $featuresAvailable[(int) $cfg->FeatureForAttrG4];
		if (!empty($cfg->FeatureForCategory1) && isset($featuresAvailable[$cfg->FeatureForCategory1])) $features[] = $featuresAvailable[(int) $cfg->FeatureForCategory1];
		if (!empty($cfg->FeatureForCategory2) && isset($featuresAvailable[$cfg->FeatureForCategory2])) $features[] = $featuresAvailable[(int) $cfg->FeatureForCategory2];
		if (!empty($cfg->FeatureForCategory3) && isset($featuresAvailable[$cfg->FeatureForCategory3])) $features[] = $featuresAvailable[(int) $cfg->FeatureForCategory3];
		if (!empty($cfg->FeatureForCategory4) && isset($featuresAvailable[$cfg->FeatureForCategory4])) $features[] = $featuresAvailable[(int) $cfg->FeatureForCategory4];
		if (!empty($cfg->FeatureForSeason) && isset($featuresAvailable[$cfg->FeatureForSeason])) $features[] = $featuresAvailable[(int) $cfg->FeatureForSeason];
		if (!empty($cfg->FeatureForCollection) && isset($featuresAvailable[$cfg->FeatureForCollection])) $features[] = $featuresAvailable[(int) $cfg->FeatureForCollection];

		if (count($features) > 0)
		{
			if (count($features) > 1)
				$js .= "$('#features-content .feature-collection').prepend('<div class=\"polaris\">⚠ Les caractéristiques ".join(', ', $features)." sont positionnées par Polaris !</div>');";
			else
				$js .= "$('#features-content .feature-collection').prepend('<div class=\"polaris\">⚠ La caractéristique ".join(', ', $features)." est positionnée par Polaris !</div>');";
		}
		
		// Application
		$style .= "#show_variations_selector { display: none !important; }";
		foreach ($fields_to_disabled as $field)
		{
			$style .= $field." { user-select: none !important; pointer-events: none !important; border: 1px solid $polarisColor !important; color: $polarisColor; background: $polarisBgColor;}\n";
			$js .= "$('".$field."').attr('tabindex', '-1');\n";
		}

		$style .= join(", ", $fields_to_hide)." { display: none !important; }\n";
		
		// Photo ?
		if ($cfg->OPT_ALLOW_I_PHOTOS)
		{
			$style .= "#product-images-container {border: 1px solid $polarisColor !important;}\n";
			$style .= "#product-images-dropzone.dropzone .dz-preview.openfilemanager { display: none !important; }\n";
			$style .= "#product-images-dropzone.dropzone .dz-preview, #product-images-container { user-select: none !important; pointer-events: none !important; }\n";
			$js .= "$('.openfilemanager').html('<i class=\"material-icons\">add_a_photo</i><div><em>Les photos des produits sont gérées depuis Polaris</em></span></div>');\n";				
		}

		$js .= "$('#form-loading').on('DOMNodeInserted', '.combination-form', function () {			
					let id = $(this).attr('data');
					let disabled = ['reference', 'quantity', 'location', 'wholesale_price', 'price', 'priceTI', 'weight', 'unity', 'ean13'];

					for (const element of disabled)
						$('#combination_' + id + '_attribute_' + element).addClass('polaris-locked');
	  			});\n";

		$output = "";
		$style .= ".polaris { color: $polarisColor; margin-bottom: 1ex;}\n";
		$style .= ".polaris-locked { user-select: none !important; pointer-events: none !important; border: 1px solid $polarisColor !important; color: $polarisColor; background: $polarisBgColor; }\n";
		if (!empty($style))
			$output .= "<style>\n".$style."</style>\n";
		if (!empty($js))
			$output .= "<script language='javascript'>\n$(document).ready(function() {".$js."});\n</script>\n";

		return $output;
	}

	public function hookActionObjectManufacturerUpdateAfter($params)
	{
		$object = $params["object"];

		foreach ([	_PS_MANU_IMG_DIR_ . $object->id.".webp", 
					_PS_MANU_IMG_DIR_ . $object->id."-*.webp",
					_PS_TMP_IMG_DIR_ . "manufacturer_mini_" . $object->id.".webp",
				] as $pattern)
		foreach (glob($pattern) as $file)
			unlink($file);
	}


	public function hookActionObjectCategoryUpdateAfter($params)
	{
		$object = $params["object"];

		foreach ([	_PS_CAT_IMG_DIR_ . $object->id.".webp", 
					_PS_CAT_IMG_DIR_ . $object->id."-*.webp",
					_PS_TMP_IMG_DIR_ . "category_" . $object->id.".webp",
					_PS_TMP_IMG_DIR_ . "category_" . $object->id."-*.webp",
				] as $pattern)
		foreach (glob($pattern) as $file)
			unlink($file);			
	}

	public function hookActionObjectCustomerAddBefore($params)
	{
		$this->hookActionObjectCustomerUpdateBefore($params);
	}

	public function hookActionObjectCustomerUpdateBefore($params)
	{
		$customer = $params["object"];
		$correctLastname = strtoupper(checkCustomerLastName($customer->lastname));
		if ($customer->lastname != $correctLastname)
			$customer->lastname = $correctLastname;

		$correctFirstname = ucwords(strtolower(checkCustomerFirstName($customer->firstname)), " -\t\r\n\f\v");
			if ($customer->firstname != $correctFirstname)
				$customer->firstname = $correctFirstname;
	}	

	/*
	private $_stores = NULL;

	public function hookActionAdminOrdersListingFieldsModifier($params)
	{
		$this->_stores = [];

		foreach (PolarisLinkStockManager::GetMappedStore() as $store)
			$this->_stores[$store["id_store"]] = $store["code_polaris"] ." - ".$store["name"];

		if (!empty($this->_stores))
		{
			$idx = 0;
			$newParams = [];
			foreach ($params["fields"] as $key => &$param)
			{
				if ($idx++ == 2)
				{
					$newParams['id_store'] = array(
						'title' => $this->l('Magasin'),
						'align' => 'center',
						'type' => 'select',
						'list' => $this->_stores,
						'filter_key' => 'a!id_store',
						'filter_type' => 'int',
						'order_key' => 'cname',
					);
				}
					
				$newParams[$key] = $param;
			}

			$params["fields"] = $newParams;
		}
	}

	public function hookActionAdminOrdersListingResultsModifier($params)
	{
		if (!empty($this->_stores))
		{
			foreach ($params["list"] as &$item)
				if (isset($this->_stores[$item["id_store"]]))
					$item["id_store"] = $this->_stores[$item["id_store"]];
		}
	}
	*/

	/*
		Réenregistrement des hooks...
	*/
	public function registerHooks($updatePosition = true)
	{
		foreach ($this->getHooks() as $hook)
			$this->registerHook($hook);

		if ($updatePosition)
		{
			$this->updatePosition(Hook::getIdByName('dashboardZoneOne'), 0, 1);
			$this->updatePosition(Hook::getIdByName('actionObjectCustomerAddBefore'), 0, 1);
			$this->updatePosition(Hook::getIdByName('actionObjectCustomerUpdateBefore'), 0, 1);
		}
	}
	
	/*
		Procédure d'installation.
		S'occupe de mettre en place les premiers hooks, le reste étant fait par
		l'application des mises à jours successives :)
	*/
	public function install()
	{
		if (!parent::install())
			return false;
		
		try
		{
			// Install admin module ...
			$tab = new Tab();
			$tab->active = 1;
			$tab->class_name = "AdminPolarisLinkVegaAutoUpdater";
			$tab->name = array();
			foreach (Language::getLanguages(true) as $lang)
				$tab->name[$lang['id_lang']] = "Mise à jour du module PolarisLink";			
			$tab->id_parent = -1;
			$tab->module = $this->name;
			$tab->add();
			
			if (!$this->upgrade_to_current())			
				throw new Exception("Erreur lors du déploiement de la base de données");
			
			$this->LoadConfig();			
			
			/* si tout le monde est passé, alors on est content ! */		
			return true;
		}
		catch (Exception $ex)
		{
			$this->_errors[] = Tools::displayError($ex->getMessage());
			$this->uninstall();
			
			return false;
		}
	}
	
	/*
		Désinstallation: on fait le ménage
	*/
	public function uninstall()
	{	
		// Uninstall Module
		if (!parent::uninstall())
			return false;
		
		// On nettoie les modules
		foreach (Tab::getCollectionFromModule($this->name) as $tab)
			$tab->delete();
			
		// On nettoie les points d'accroche
		foreach ($this->getHooks() as $hook)
			$this->registerHook($hook);
			
		return true;	
	}

	/*
		Envoi les métriques à VEGA Informatique à des fins de maintenance et de licences
	*/
	public function sendMetrics($force = false)
	{
		try
		{
			// TODO
			
			/*$lastMetricsSent = Configuration::get("POLARISLINK_METRICS_SENT");		
			if ($force || $lastMetricsSent < date("Y-m-d") )
			{				

				// Métriques envoyées, même s'il y a eu une erreur !
				Configuration::updateValue("POLARISLINK_METRICS_SENT", date("Y-m-d"));
			}*/
		}
		catch (\Exception $ex)
		{
			// Pas grave
			//echo $ex->getMessage();
		}		
	}
	
	/*
		Signale une erreur à afficher en Web2.0 dans l'interface
		lors de son rendu.
		Ne déclenche pas l'affichage immédiat de l'erreur mais simplement programme
		son rendu.
		UNIQUEMENT valable lors de l'installation/initialisation !
	*/
	public function install_error($msg)
	{
		$this->_errors[] = $msg;
	}
	
	public static function IsHostedByVega()
	{
		return isset($_SERVER["VEGA_HOSTING"]);
	}

	public function GetReexportedOrders()
	{
		$list = json_decode(Configuration::get("POLARISLINK_REEXPORT_ORDERS"), true);
		if (empty($list))
			$list = array();

		return $list;
	}

	public function SetOrderRexported()
	{
		Configuration::updateValue("POLARISLINK_REEXPORT_ORDERS", "");
	}

	public function ReexportOrder($orderId)
	{
		$list = $this->GetReexportedOrders();
		$list[(int) $orderId] = (int) $orderId;
		Configuration::updateValue("POLARISLINK_REEXPORT_ORDERS", json_encode($list));
	}
	
	private function getMultipleParams($paramId, $possibleChoices, $possibleChoiceKey)
	{
		$checked = array();
		foreach ($possibleChoices as $choice)
		{
			if (Tools::getValue($paramId.'_'.$choice[$possibleChoiceKey]))
				$checked[] = $choice[$possibleChoiceKey];
		}
		return $checked;
	}
	
	public function getAuthTokenStr()
	{
		$ip = "";
		foreach ([ "HTTP_X_FORWARDED_FOR", "HTTP_X_REMOTE_IP", "REMOTE_ADDR" ] as $v)
		{
			if (isset($_SERVER[$v]) && !empty($_SERVER[$v]))
			{
				$ip = $_SERVER[$v];
				break;
			}
		}

		$tk = "PolarisLinkAPI;".$ip.";";
		return Tools::getAdminToken($tk);
	}

	/**
	 * Sauve le .htaccess
	*/
	public function saveHtAccess()
	{
		$path = _PS_ROOT_DIR_ . '/.htaccess';

        // Check current content of .htaccess and save all code outside of prestashop comments
        $specific_before = $specific_after = '';
        if (file_exists($path)) {
            $old_content = file_get_contents($path);
            if (preg_match('#^(.*)\# ~~pl_start~~.*\# ~~pl_end~~[^\n]*(.*)$#s', $old_content, $m)) {
                $specific_before = $m[1];
                $specific_after = $m[2];
            }
			else
				$specific_after = "\n".$old_content;
        }
		        
        // Génération du nouveau contenu
		$new_content = $specific_before;
        $new_content .= "# ~~pl_start~~ Do not remove this comment, PolarisLink will keep automatically the code outside this comment when .htaccess will be generated again\n";
        $new_content .= "# .htaccess automaticaly generated by PolarisLink\n\n";

		$new_content .= "# Ajout du support des Webp\n";
		$new_content .= "<Files \"*.webp\">\n  AddType image/webp .webp\n  Require all granted\n</Files>\n\n";

		if (!function_exists("Tools::getDomains"))
		{			
			$domains = array();
			foreach (ShopUrl::getShopUrls() as $shop_url) {
				/** @var ShopUrl $shop_url */
				if (!isset($domains[$shop_url->domain])) {
					$domains[$shop_url->domain] = array();
				}

				$domains[$shop_url->domain][] = array(
					'physical' =>    $shop_url->physical_uri,
					'virtual' =>    $shop_url->virtual_uri,
					'id_shop' =>    $shop_url->id_shop
				);

				if ($shop_url->domain == $shop_url->domain_ssl) {
					continue;
				}

				if (!isset($domains[$shop_url->domain_ssl])) {
					$domains[$shop_url->domain_ssl] = array();
				}

				$domains[$shop_url->domain_ssl][] = array(
					'physical' =>    $shop_url->physical_uri,
					'virtual' =>    $shop_url->virtual_uri,
					'id_shop' =>    $shop_url->id_shop
				);
			}
		}
		else
			$domains = Tools::getDomains();

		$medias = [
			Configuration::getMultiShopValues('PS_MEDIA_SERVER_1'),
			Configuration::getMultiShopValues('PS_MEDIA_SERVER_2'),
			Configuration::getMultiShopValues('PS_MEDIA_SERVER_3'),
		];

        $media_domains = '';
        foreach ($medias as $media) {
            foreach ($media as $media_url) {
                if ($media_url) {
                    $media_domains .= '  RewriteCond %{HTTP_HOST} ^' . $media_url . '$ [OR]' . PHP_EOL;
                }
            }
        }

		$new_content .= "<IfModule mod_rewrite.c>\n";

		// In case the user hasn't rewrite mod enabled
		$new_content .= "  # Allow HTTP AUTH\n";
		$new_content .= "  RewriteEngine on\n  RewriteCond %{HTTP:Authorization} ^(.*)\n  RewriteRule . - [E=HTTP_AUTHORIZATION:%1]\n\n";
		
		if (Configuration::get("POLARISLINK_USE_WEBP"))
		{
			foreach ($domains as $domain => $list_uri) {
				// As we use regex in the htaccess, ipv6 surrounded by brackets must be escaped
				$domain = str_replace(['[', ']'], ['\[', '\]'], $domain);

				$domain_rewrite_cond = '  RewriteCond %{HTTP_HOST} ^' . $domain . '$' . PHP_EOL;
				foreach ($list_uri as $uri) 
				{
					$new_content .= "  #Domain: " . $domain . PHP_EOL;
					if (Shop::isFeatureActive()) {
						$new_content .= "  RewriteCond %{HTTP_HOST} ^" . $domain . '$' . PHP_EOL;
					}
					$new_content .= "  RewriteRule . - [E=REWRITEBASE:".$uri['physical']."]\n\n";

					$new_content .= "  # des *.webp de substitution sont servis en lieu et place des .jpeg / ou générés à la volée !\n";
					$new_content .= "  # mais seulement pour les navigateurs qui le supportent !\n";

					$new_content .= $media_domains;
					$new_content .= $domain_rewrite_cond;
					$new_content .= "  RewriteCond %{HTTP_ACCEPT} image/webp"."\n";
					$new_content .= "  RewriteRule ^img/(.*).jpg$ %{ENV:REWRITEBASE}img/$1.webp [L]\n";

					$new_content .= $media_domains;
					$new_content .= $domain_rewrite_cond;
					$new_content .= "  RewriteCond \"%{REQUEST_FILENAME}\" \\.webp$\n  RewriteCond \"%{REQUEST_FILENAME}\" !-f\n  RewriteRule ^.*$ %{ENV:REWRITEBASE}index.php?media=%{REQUEST_FILENAME} [L]\n";
					$new_content .= "\n";
				}
			}
		}

		$new_content .= "</IfModule>\n\n";
		
		$new_content .= '# ~~pl_end~~ Do not remove this comment, Prestashop will keep automatically the code outside this comment when .htaccess will be generated again';

		$new_content .= $specific_after;
		
		if ($new_content != $old_content)
			file_put_contents($path, $new_content);

		return true;
	}

	/**
		Converti une image en WebP
	*/
	public function convertImgToWebp($file, $compression_quality = 80)
	{
		$output_file = substr($file, 0, strrpos($file, '.')) . ".webp";

		// check if file exists
		if (!file_exists($file)) {
			return false;
		}

		// If output file already exists return path
		//$output_file = $file . '.webp';
		if (file_exists($output_file)) {
			return $output_file;
		}

		$attrs = getimagesize($file);

		if (function_exists('imagewebp')) 
		{
			switch ($attrs["mime"]) {
				case 'image/jpeg':
				case 'image/jpg':
					$image = imagecreatefromjpeg($file);
					break;

				case 'image/png':
					$image = imagecreatefrompng($file);
					imagepalettetotruecolor($image);
					imagealphablending($image, true);
					imagesavealpha($image, true);
					break;

				case 'image/gif':
					$image = imagecreatefromgif($file);
					break;
				default:
					return false;
			}
			// Save the image
			$result = imagewebp($image, $output_file, $compression_quality);

			// Free up memory
			imagedestroy($image);

			if ($result)
				return $output_file;
		} 
		
		if (class_exists('Imagick')) 
		{
			$image = new Imagick();
			$image->readImage($file);

			if ($attrs["mime"] === 'png') {
				$image->setImageFormat('webp');
				$image->setImageCompressionQuality($compression_quality);
				$image->setOption('webp:lossless', 'true');
			}

			if ($image->writeImage($output_file))
				return $output_file;
		}

		return false;
	}

	/**
	 * Interception variables templates
	 */
	public function hookActionFrontControllerSetVariables($params)
	{
		// Interception des variables templates !
		$vars = $params['templateVars'];

		if ($this->context->controller instanceof ProductController)
		{
			// On intercepte les variables du produit
			$product_id = $this->context->smarty->getTemplateVars('product')->id;
			if (!empty($product_id) && Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"))
			{
				$productsList = $this->loadRelatedProductsInfo(array($product_id));

				$products = array();
				foreach ($productsList["products"][$product_id] as $product)
				{
					if (!$product->active)
						continue;

					$products[] = $product;
				}

				if (count($products) > 0)
				{
					// Identification du produit par défaut
					// Le produit par défaut est l'original (le plus petit id)
					$canonical_product = $products[0];
					// Au passage, on charge notre produit actuel
					$current_product = null;
					
					foreach ($products as $product)
					{
						if ($canonical_product->id > $product->id)
							$canonical_product = $product;

						if ($product->id == $product_id)
							$current_product = $product;
					}

					// Pas de lien canonique si le produit actuel EST le produit canonique
					// ou si la description est différente
					if ($canonical_product->id == $product_id ||
						($current_product && trim($canonical_product->description) != trim($current_product->description)))
						$canonical_product = null;

					// Si ce n'est pas le produit en cours ...
					if ($canonical_product)
					{
						$url = $this->context->link->getProductLink($canonical_product, null, null, null, null, null, null);
						header( sprintf('Link: <%s>; rel="canonical"', $url ));	
						$params['templateVars']['page']['canonical'] = $url;
					}
				}
			}
		}
	}
	
	public function hookActionDispatcher($params)
	{		
		if ($params["controller_class"] != "PageNotFoundController")
			return false;

		// Conversion webp ?
		if (isset($_REQUEST["media"]))
		{			
			$mediaFile = $_REQUEST["media"];
			if (strlen($mediaFile) > 0 && (substr($mediaFile, 0, 1) == "/" || substr($mediaFile, 0, 1) == "\\") &&
				substr($mediaFile, 0, strlen(_PS_IMG_DIR_)) != _PS_IMG_DIR_)
			{
				throw new Exception("Bad path");
			}

			$mediaFile = substr($mediaFile, 0, strrpos($mediaFile, '.')). ".jpg";
			$mediaFile = $this->convertImgToWebp($mediaFile);
			if (file_exists($mediaFile))
			{
				header("Content-Type: image/webp");
				header("X-Web-P: On-The-Fly");
				print @file_get_contents($mediaFile);
				die();
			}
		}

		$request = $_SERVER["REQUEST_URI"];
	
		$db = Db::getInstance(_PS_USE_SQL_SLAVE_);
		if ($results = $db->ExecuteS("SELECT new_url FROM `"._DB_PREFIX_."polarislink_ips_redir` WHERE '".pSQL($request)."' LIKE old_url"))
		{
			foreach ($results as $result)
			{
				if (!empty($result["new_url"]))
				{
					// Ok, redirection !
					$to = Tools::getShopProtocol().Tools::getShopDomainSsl().$result["new_url"];
					
					header("Location: ".$to, true, 301); 
					header("Content-Type: text/html; charset=utf-8");
					echo '<html><head></head><body>Vous allez être redirigé vers la page <a href="'.$to.'">'.$to.'</a>.</body></html>';
					
					die();
				}
			}
		}
		
		$root_request = substr($request, strlen(__PS_BASE_URI__));
		if (substr($root_request, 0, 8) == "marques/")
		{
			// C'est une marque :
			$url_data = explode('/', substr($root_request, 8));

			// Chargement de la marque
			$manufacturer_id = Manufacturer::getIdByName($url_data[0]);
			if ($manufacturer_id)
			{
				// On reconstruit l'URL souhaité
				$link = new Link();
				$real_url = $link->getManufacturerLink(new Manufacturer($manufacturer_id));
				// On coupe à la racine
				$real_url = substr($real_url, strpos($real_url, '/', 7));

				if (count($url_data) > 1)
					$real_url .= "?".urlencode($url_data[1])."=".urlencode($url_data[2]);

				$_SERVER["REQUEST_URI"] = $real_url;
				$_GET['controller'] = null;
				Dispatcher::$instance = null;
				Dispatcher::getInstance()->dispatch();

				die();
			}			
		}
		
		return false;
	}

	/*
		On agrémente le panneau d'administration des commandes avec nos 
		fonctions.
	*/
	public function hookActionGetAdminOrderButtons(array $params)
	{
		$id_order = (int) $params['id_order'];

		if (isset($_REQUEST["p_action"]) && $_REQUEST["p_action"] == "reexport")
			$this->ReexportOrder($id_order);

		$rexport = $this->GetReexportedOrders();
		
		/** @var \Symfony\Bundle\FrameworkBundle\Routing\Router $router */
        $router = $this->get('router');

        /** @var \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButtonsCollection $bar */
        $bar = $params['actions_bar_buttons_collection'];

		if (isset($rexport[$id_order]))
		{
			$bar->add(new \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButton(
				'btn-link disabled', 
				['href' => '#'], 
				'à réexporter vers Polaris'
			));
		}
		else
		{
        	$bar->add(new \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButton(
                	'btn-action', 
					['href' => str_replace("&p_action=reexport", "", $_SERVER["REQUEST_URI"]).'&p_action=reexport'], 
					'Réexporter vers Polaris'
            	));
		}

		return "";
	}

	/*
		Construction du sélecteur visuel
	*/
	public function buildVisualProductsSelector($id_product)
	{
		// Chargement des produits liés ...
		$context = Context::getContext();		

		$excludeCurrent = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT");
		$onlyWithStocks = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_STOCK");
		
		$results = $this->loadRelatedProductsInfo(array($id_product));

		$products = array();
		foreach ($results["products"][$id_product] as $product)
		{
			if (!$product->active)
				continue;

			if ($onlyWithStocks && $product->quantity <= 0)
				continue;

			if ($excludeCurrent && $product->id == $id_product)
				continue;

			$products[] = $product;
		}

		if (count($products) > 0)
		{			
			echo '<style>';
			echo Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS");
			echo '</style>';

			echo '<div class="polarislink-critselectblock">';
			//echo '<p class="polarislink-icien"><span class="lbl">'.Translate::getModuleTranslation("polarislink", "Présenté ici en :", "").'</span><span class="vle"></span></p>';
			/*echo '<p class="polarislink-existeen"><span class="lbl">'.
				($excludeCurrent ? 	Translate::getModuleTranslation("polarislink", "Existe également en :", "") : 
									Translate::getModuleTranslation("polarislink", "Existe en :", "")).'</span></p>';*/
			echo '<p class="polarislink-existeen"><span class="lbl">'.
				($excludeCurrent ? 	$this->l("Existe également en :") : 
									$this->l("Existe en :")).'</span></p>';
			echo '<ul class="polarislink-selecteur">';
			
			$libelle = null;
			foreach ($products as $product)
			{
				if ($libelle == null)
					$libelle = $product->name;
				if ($libelle != $product->name)
				{
					$libelle = null;
					break;
				}
			}
			
			foreach ($products as $product)
			{
				// Si $libelle est vide, alors le nom est descriminant, sinon on prend la référence
				$product_title = $libelle ? htmlspecialchars($product->reference) : htmlspecialchars($product->name);
				echo '<li>';
				echo '<a '.($product->id == $id_product ? 'class="active"' : '').
						'href="'.$context->link->getProductLink($product, null, null, null, null, null, Product::getDefaultAttribute($product->id, 1)).'" '.
						'title="'.$product_title.'">';

				$cover = $product->getCover($product->id);
				if (!empty($cover) && isset($cover['id_image']) && !empty($cover['id_image']))
					$img_url = $context->link->getImageLink($product->link_rewrite, $cover['id_image'], 'selection');
				else
					$img_url = "/img/p/fr-default-selection.jpg";
				echo '<img itemprop="image" src="'.$img_url.'" title="'.$product_title.'" alt="'.$product_title.'" />';				
				echo '</a>';
				echo '</li>';
			}

			echo '</ul>';
			echo '<div class="polarislink-selecteur-end"></div>';
			echo '</div>';
		}
	}

	public function hookDisplayProductPriceBlock($params)
	{
		if (isset($params["type"]) && $params["type"] != "after_price")
			return;

		if (!Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_USE"))
			return;	// Génération non désirée

		if (is_array($params["product"]))
			$this->buildVisualProductsSelector($params["product"]["id_product"]);
		else
			$this->buildVisualProductsSelector($params["product"]->id);
	}

	public function hookActionPolarisLinkProductSelector($params)
	{
		$this->buildVisualProductsSelector($params["product_id"]);
	}

	public function hookDisplayHeader($params)
	{
		$ga_tag = Configuration::Get("POLARISLINK_GOOGLE_TAGS", "");
		if (!empty($ga_tag))
		{
			$code = "";
			$code .= "<!-- Global site tag (gtag.js) - Google Analytics -->
				<script async src=\"https://www.googletagmanager.com/gtag/js?id=".$ga_tag."\"></script>
				<script>
				  window.dataLayer = window.dataLayer || [];
				  function gtag(){dataLayer.push(arguments);}
				  gtag('js', new Date());
				
				  gtag('config', '".$ga_tag."');
				</script>
				";		

			return $code;
		}
		else
			return "";
	}

	// Obtient l'URL de progression
	public function getCheckProgressUrl($params="")
	{
		return $this->context->link->getBaseLink().
						'?fc=module&module=polarislink&controller=api_checkprogress&all=1&pl_auth_tk='.$this->getAuthTokenStr().$params;
	}

	public function hookDashboardZoneOne($params)
	{
		$adminAddon = '';
		if (stripos($this->context->employee->email, "@vega-info.fr") !== false)
			$adminAddon = '<p>'.
							'[&nbsp;<a href="'.$this->context->link->getAdminLink("AdminModules") . '&configure=polarislink&tab_module=administration&module_name=polarislink">Configuration de la passerelle</a>&nbsp;]'.
							' – [&nbsp;<a href="'.$this->context->link->getAdminLink("AdminLogs") . '&logOrderby=id_log&logOrderway=desc">Journaux</a>&nbsp;]'.
							'</p>';

		$output = '<div class="panel">
						<h3>Synchronisation Polaris v'.PolarisLink::GetFullVersion().'</h3>
						<p id="plink-loading" style="text-align: center;">
							<i class="icon icon-refresh icon-spin"></i>&nbsp;&nbsp;Interrogation serveur en cours...
						</p>
						<p id="plink-in-progress" style="display: none;">
							Le traitement du fichier de synchronisation est en cours...
						</p>
						<p id="plink-stalled" style="display: none;">
							Le traitement du fichier de synchronisation est arrêté&nbsp;!? <br>'.

							((stripos($this->context->employee->email, "@vega-info.fr") !== false) ? 
								'[&nbsp;<a target="_blank" href="'.$this->context->link->getModuleLink('polarislink','api_run', array('wait' => 1, 'debug' => 1, 'pl_auth_tk' => $this->getAuthTokenStr())).'
								">Debug</a>&nbsp;]' 
								: '').

							'[&nbsp;<a href="javascript: polarisLinkDoJobs();">Reprendre</a>&nbsp;]
							[&nbsp;<a href="javascript: polarisLinkClearJob();">Supprimer</a>&nbsp;]
						</p>
						<p id="plink-err" style="display: none;">
						</p>
						<p id="plink-finished" style="display: none;">
							Aucun traitement en cours.<br>							
						</p>
						<p id="plink-cron" style="display: none;">
							Fichiers en attente de prise en charge par les tâches automatisées du système.<br>
						</p>
						<div id="plink-infos" style="display: none;">
							<div id="plink-cprogress" class="progress progress-striped active">
								<div id="plink-progress" class="progress-bar progress-bar-success" role="progressbar" aria-valuenow="60" aria-valuemin="0" aria-valuemax="100" style="width: 0%;">
								</div>
							</div>
							<div id="plink-stats">
							</div>
						</div>
						'.$adminAddon.'
					</div>
					'.
					"<script type=\"text/javascript\">
						function polarisLinkDoJobs()
							{
								$.ajax({
									  url: '".$this->context->link->getModuleLink('polarislink','api_run', array('pl_auth_tk' => $this->getAuthTokenStr()))."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-cron').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkClearJob()
						{
								$.ajax({
									  url: '".$this->context->link->getModuleLink('polarislink','api_downloadJob', array('pl_auth_tk' => $this->getAuthTokenStr(), 'clear' => '1'))."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-cron').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkDismissErrors()
							{
								$.ajax({
									  url: '".$this->getCheckProgressUrl('&dismiss_errors=1')."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-cron').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkRefresh(do_not_recall)
						{
							$.ajax({
								  url: '".$this->getCheckProgressUrl()."',
								}).complete(function(data, textStatus) {
								  
								  $('#plink-loading').hide();

								  if (data.responseJSON === undefined && data.working != undefined)
									data = { responseJSON: data };	// Fix 1.7.7.1

								  if (data.responseJSON != undefined)
								  {
									  let pdone = data.responseJSON.todo == 100 ? 0 : Math.round(data.responseJSON.done * 100 / data.responseJSON.todo);

									  let stats = 'Détails de l\'opération en cours : <ul>';

									  for (let prop in data.responseJSON.details)
									  {
										stats += '<li>';										
										if (data.responseJSON.details[prop].done >= data.responseJSON.details[prop].cnt)
											stats += '<strong>';
									  	stats += data.responseJSON.details[prop].done + '/' + data.responseJSON.details[prop].cnt + ' ' + data.responseJSON.details[prop].title.toLowerCase() + ' traité(es).';
										  if (data.responseJSON.details[prop].done >= data.responseJSON.details[prop].cnt)
										  stats += '</strong>';
										stats += '</li>'
									  }
									  
									  stats += '<li>' + data.responseJSON.err + ' erreurs</li>';

									  stats += '</ul>';

									  //$('#plink-stats').html(data.responseJSON.done + '/' + data.responseJSON.todo);
									  $('#plink-stats').html(stats);
									  
									  $('#plink-progress').html(pdone + '&nbsp;%');
									  $('#plink-progress').css('width', pdone + '%');
									  
									  $('#plink-err').hide();									  
									  
									  if (data.responseJSON.todo == 0 || data.responseJSON.done == data.responseJSON.todo)
									  {
										let finishedText = 'Aucun traitement en cours.<br>';
										if (data.responseJSON.last_sync != null)
										{
											let dt = new Date(data.responseJSON.last_sync);											
											finishedText += 'Dernière synchronisation le ' + dt.toLocaleDateString() + ' à ' + dt.toLocaleTimeString() + '.<br>';
										}
										if (data.responseJSON.err > 0)
										{
											finishedText += '<br><div class=\"alert alert-danger\"><strong>Il s\'est produit ' + data.responseJSON.err + ' erreurs</strong> lors des dernières synchronisations : <br>' +
															'[&nbsp;<a href=\"".$this->context->link->getAdminLink("AdminLogs", array("logOrderby" => "id_log", "logOrderway" => "desc"))."\"><i class=\"icon icon-question-circle\"></i> voir</a>&nbsp;] — ' +
															'[&nbsp;<a href=\"javascript: polarisLinkDismissErrors();\"><i class=\"icon icon-ok-sign\"></i> marquer comme vu</a>&nbsp;]</div>';
										}									
											
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-cron').hide();
										$('#plink-finished').html(finishedText);
										$('#plink-finished').show();
									  }
									  else
									  {
										  $('#plink-infos').show();
										  
										  $('#plink-finished').hide();
										  if (data.responseJSON.working)
										  {
											$('#plink-cprogress').addClass('progress-striped');
											$('#plink-cprogress').addClass('active');
											$('#plink-progress').addClass('progress-bar-success');
											$('#plink-progress').removeClass('progress-bar-warning');
											$('#plink-in-progress').show();
											$('#plink-stalled').hide();
											$('#plink-cron').hide();
										  }
										  else
										  {
											$('#plink-cprogress').removeClass('progress-striped');
											$('#plink-cprogress').removeClass('active');
											$('#plink-progress').removeClass('progress-bar-success');
											$('#plink-progress').addClass('progress-bar-warning');
											
											$('#plink-in-progress').hide();
											if (data.responseJSON.cron == 1)
											{
												$('#plink-infos').hide();
												$('#plink-cron').show();	
											}
											else
											{
												$('#plink-stalled').show();
											}
										  }
									  }
								  }
								  else
								  {
									  $('#plink-err').html('err ' + data.statusText + ' : ' + data.responseText);
									  $('#plink-err').show();
								  }
								  
								  if (do_not_recall === undefined || !do_not_recall)
									window.setTimeout(polarisLinkRefresh, 5000);
								});
						}
						
						polarisLinkRefresh();
					</script>";
		return $output;
	}
	
	/*
		Essayes la clé fournie
	*/
	private function test401($key)
	{
		$auth = base64_encode(":".$key);
		$context = stream_context_create([
			"http" => [
				"header" => "Authorization: Basic $auth"
			]
		]);

		$headers = get_headers($this->context->link->getModuleLink('polarislink','api_info', array()), 0, $context);
		if ($headers && isset($headers[0]) && strpos($headers[0], '401'))
			return false;
		
		return true;
	}

	/* 
		Obtient l'ID de la dernière commande à ne jamais renvoyer à Polaris
	*/
	public function getOrderBarrier()
	{
		// On évite de rappatrier toutes les ventes !!
		$orderBarrier = Configuration::get("POLARISLINK_ORDER_BARRIER", "");
		if ($orderBarrier == "")
		{
			// Détection de la barrière
			$orderBarrier = 1;

			$installDate = Configuration::get("POLARISLINK_INSTALL_DATE", "");
			if (!empty($installDate))
			{
				// S'il s'agit d'une barrière ancienne, on retrouve le numéro de commande via la date de barrière
				$db = Db::getInstance();
				$results = $db->executeS(sprintf("SELECT DISTINCT COALESCE(MAX(o.id_order), 1) AS id_order FROM "._DB_PREFIX_."orders o WHERE o.date_add <= '%s'", $installDate), true, false);
				if ($results === false)
					throw new PrestaShopDatabaseException($db->getMsgError());
				if (count($results) > 0)
					$orderBarrier = (int) $results[0]["id_order"];
			}
			Configuration::updateValue("POLARISLINK_ORDER_BARRIER", $orderBarrier);
		}

		return $orderBarrier;
	}
	
	/*
		Configuration Page
	*/
	public function getContent()
	{
		$this->LoadConfig(true);

		$isVega = stripos($this->context->employee->email, "@vega-info.fr") !== false;

		// Traduction
		$this->l("Soldes");
		$this->l('Existe en :');
		$this->l('Existe également en :');

		$output = "<style>
				.pull-left { margin-left: 4px; }
				.expert { display: none; background-color: #ffe7e1 !important; padding: 1ex 0 !important; opacity: 0.6; }
			</style>
			<script type=\"text/javascript\">
			/* Code generator for Affiliation and vouchers */
			function VEGAGenKey(eltName, size)
			{
					getE(eltName).value = '';
					/* There are no O/0 in the codes in order to avoid confusion */
					var chars = '123456789ABCDEFGHIJKLMNPQRSTUVWXYZ';
					for (var i = 1; i <= size; ++i)
							getE(eltName).value += chars.charAt(Math.floor(Math.random() * chars.length));
			}
		</script>";		
				
		//$id_lang = Configuration::get('PS_LANG_DEFAULT');
		$id_lang = (int) $this->context->language->id;

		// Chargement des stores
		$stores = PolarisLinkStockManager::GetMappedStore($id_lang, true);
		
		/* Visilibités de stock */
		$list_visibilities = array(
			"query" => [
				["id" => "", "name" => "Ne pas changer la visilibilité des produits"],
				["id" => "both", "name" => "Partout"],
				["id" => "none", "name" => "Nulle part"],
				["id" => "search", "name" => "Uniquement dans les résultats de recherche"],
				["id" => "catalog", "name" => "Uniquement dans le catalogue"],				
			],
			"id" => "id",
			"name" => "name"
		);

		/* Options */
		$opts = array(
						"OPT_REFS_CACHE" => array(
							"t" => "Utiliser le cache mémoire des références",
							"d" => "Cette option permet de charger préalablement toutes les références en mémoire afin de diminuer fortement les requêtes au serveur de bases de données et ainsi améliorer considérablement les temps de traitement. Désactiver cette option si vous ne disposez pas assez de mémoire pour vous permettre de charger toutes les références.",
							"e" => 1,
						),
						"OPT_ACTIVE_PUBLISHED" => array(
														"t" => "Active les produits publiés",
														"d" => "Cette option permet d'activer les produits publiés automatiquement",
														"e" => 1,
													),
						"OPT_UNPUBLISH_COOLDOWN" => array(
														"t" => "Désactive les produits dépubliés en différé",
														"d" => "Cette option permet de désactiver les produits dépubliés en différé en vidant leur stock afin de laisser le temps aux exportateurs de récupérer les informations nécessaires à leur retrait. A activer si vous êtes reliés à Amazon ou Spartoo.",
														"e" => 1,
													),
						"OPT_ALLOW_I_PHOTOS" => array(
														"t" => "Permettre l'importation des photos",
														"d" => "Si cette option est désactivée les photos transmises seront ignorées",
													),
						"OPT_ALLOW_I_NAME" => array(
														"t" => "Permettre la modification des noms des produits",
														"d" => "Si cette option est désactivée les noms transmis seront ignorées",
						),
						"OPT_ALLOW_I_PRODUCT_LINK" => array(
							"t" => "Forcer le recalcul du slug/rewrite du produit",
							"d" => "Si cette option est activée, le slug du produit sera recalculé à chaque synchronisation du produit (déconseillé)",
						),
						"OPT_ALLOW_I_MDESCRIPTIONS" => array(
														"t" => "Permettre la modification des descriptions méta des entités",
														"d" => "Si cette option est désactivée les descriptions transmises seront ignorées",
						),
						"OPT_ALLOW_I_DESCRIPTIONS" => array(
														"t" => "Permettre la modification des descriptions des entités",
														"d" => "Si cette option est désactivée les descriptions transmises seront ignorées",
													),
													
						"OPT_ALLOW_I_CATALOG" => array(
														"t" => "Permettre l'importation du catalogue",
														"d" => "Si cette option est désactivée, les modifications du catalogue produits seront ignorées",
													),
						"OPT_ALLOW_I_GENDERS" => array(
														"t" => "Permettre la création des titres de civilités",
														"d" => "Si cette option est désactivée, les nouvelles civilités ne seront pas créés sous Prestashop",
													),
						"OPT_ALLOW_I_CUSTOMERS" => array(
														"t" => "Importer les clients depuis Polaris",
														"d" => "Si cette option est désactivée, les importations de clients seront ignorées",
													),
						"OPT_ALLOW_E_CUSTOMERS" => array(
														"t" => "Exporter les clients vers Polaris",
														"d" => "Si cette option est désactivée, les requêtes de données sur les clients seront rejetées",
													),	
						"OPT_ALLOW_E_MK_CUSTOMERS" => array(
														"t" => "Exporter les clients marketplace (Amazon) vers Polaris",
														"d" => "Si cette option est désactivée, les clients provenants des marketplaces ne seront pas transmis à Polaris",
													),	
						"OPT_ALLOW_E_G_CUSTOMERS" => array(
														"t" => "Exporter les clients sans compte (invités) vers Polaris",
														"d" => "Si cette option est désactivée, les clients provenants des marketplaces ne seront pas transmis à Polaris",
													),	
						"OPT_ALLOW_E_SALES" => array(
														"t" => "Permettre l'exportation des ventes",
														"d" => "Si cette option est désactivée, les requêtes de données sur les ventes seront rejetées",
													),
						"OPT_ALLOW_E_SALES_OTHER_CURRENCIES" => array(
														"t" => "Permettre l'exportation des ventes dans une autre devise que celle gérée par le pont",
														"d" => "Cette option permet d'autoriser le pont à exporter toutes les commandes, y compris celles qui ne sont pas dans la monnaie gérée par ce lien.",
						),
						"OPT_ALLOW_I_STOCKS" => array(
														"t" => "Permettre la mise à jour du stock",
														"d" => "Si cette option est désactivée, les tentatives de mise à jour du stock seront ignorées",
													),
						"OPT_ALLOW_I_SALE_PRICES" => array(
														"t" => "Permettre la modification des prix de vente",
														"d" => "Si cette option est désactivée, les tentatives de modification des prix de vente seront ignorées",
													),
						"OPT_ALLOW_I_PROMOS" => array(
														"t" => "Permettre l'importation des promotions",
														"d" => "Si cette option est désactivée, les tentatives d'importation des prix promotionnels seront ignorées",
													),
						"OPT_PAYMENT_MODULE" => array(
														"t" => "Exporter le nom du module de paiement au lieu du libellé",
														"d" => "Si cette option est activée, exporte le nom du module utilisé pour le paiement en lieu et place du libellé de ce dernier",
														"e" => 1
														),
						"OPT_ALLOW_I_VOUCHERS_ASSETS" => array(
							"t" => "Permettre l'importation des bons d'avoir",
							"d" => "Si cette option est désactivée, les importations de bons de ce type seront ignorées",
						),
						"OPT_ALLOW_I_VOUCHERS_OFFERS" => array(
							"t" => "Permettre l'importation des bons cadeaux et de remises",
							"d" => "Si cette option est désactivée, les importations de bons de ce type seront ignorées",
						),
													
						"OPT_USE_ADV_STOCK" => array(
														"t" => "Utiliser le stock avançé si disponible",
														"d" => "Utiliser la gestion des stocks avançée pour les produits gérés par cette passerelle si elle est activée",
														"e" => 1,
													),
						"OPT_USE_EAN_STOCK" => array(
														"t" => "Permettre la maj des stocks par EAN fournisseur",
														"d" => "Permet la mise à jour des stocks à partir des codes barres fournisseur (GENCODE), même si ce ne sont pas des produits gérés par Polaris",
													),
						"OPT_PROMO_AMOUNT" => array(
														"t" => "Importer les promotions en valeur plutôt qu'en pourcentage",
														"d" => "Par défaut, le pont importe vos promotions en pourcentage, ce qui peut causer des erreurs d'arrondis mais permet l'affichage d'un % sur les fichiers produits.".
																"Si vous désirez importer vos promotions et soldes en numéraire et afficher l'économie réalisée, cochez cette option.",
													),
						"OPT_USE_ARCHIVES" => array(
														"t" => "Archivage des importations",
														"d" => "Cette option permet de garder les fichiers soumis pour l'importation des données plutôt que de les supprimer",
														"e" => 1,
													),
						"OPT_NO_SYMLNK_IMG" => array(
														"t" => "Utiliser la copie des photos",
														"d" => "Par défaut, le pont fait la liaison de vos photos Prestashop avec celle importées en utilisant des liens symboliques. Si votre système ".
																"ne les supporte pas, activez cette option pour travailler avec des copies, vos photos occuperont alors le double de stockage.",
														"e" => 1,
													),
					);		

		$suppliers = Supplier::getSuppliers(false, 0, false);
				
		$shops = array();
		$shopGroups = array();
		foreach (ShopGroup::getShopGroups() as $group)
			$shopGroups[$group->id] = array('name' => $group->name);
			
		foreach (Shop::getShops() as $shop)
		{
			if (!isset($shopGroups[$shop["id_shop_group"]]["shops"]))
				$shopGroups[$shop["id_shop_group"]]["shops"] = array();
			$shopGroups[$shop["id_shop_group"]]["shops"][] = $shop;
		}
		
		foreach ($shopGroups as $id_group => $group)
		{
			$shops[] = array('id_shop' => '*'.$id_group, 'name' => $group['name']);
			foreach ($group["shops"] as $shop)
				$shops[] = array('id_shop' => $shop["id_shop"], 'name' => '    '.$shop["name"]);
		}

		$categoriesOrFeatures = [
			[ "t" => "catégories Polaris n°1", "k" => "CATEGORY1" ],
			[ "t" => "catégories Polaris n°2", "k" => "CATEGORY2" ],
			[ "t" => "catégories Polaris n°3", "k" => "CATEGORY3" ],
			[ "t" => "catégories Polaris n°4", "k" => "CATEGORY4" ],
			[ "t" => "saisons Polaris", "k" => "SEASON" ],
			[ "t" => "collections Polaris", "k" => "COLLECTION" ],						
		];
		
		// On évite de rappatrier toutes les ventes !!
		$orderBarrier = $this->getOrderBarrier();

		if (Tools::isSubmit('submit'.$this->name))
		{
			$action = Tools::getValue('PPL_VEGA_ACTION');
			
			// Ok étape 1, on sauve
			$reload = false;
			
			if ($isVega)
				Configuration::updateValue("POLARISLINK_GOOGLE_TAGS", Tools::getValue("POLARISLINK_GOOGLE_TAGS"));

			Configuration::updateValue("POLARISLINK_USE_WEBP", Tools::getValue("POLARISLINK_USE_WEBP"));
			Configuration::updateValue("POLARISLINK_AUTOCLEAN_STATS", Tools::getValue("POLARISLINK_AUTOCLEAN_STATS"));
			Configuration::updateValue("POLARISLINK_CRON_DATAUPDATE", Tools::getValue("POLARISLINK_CRON_DATAUPDATE"));

			Configuration::updateValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS", Tools::getValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS"));
			if ($orderBarrier != Tools::getValue("POLARISLINK_ORDER_BARRIER"))
			{
				Configuration::updateValue("POLARISLINK_ORDER_BARRIER", (int) Tools::getValue("POLARISLINK_ORDER_BARRIER"));
				$orderBarrier = $this->getOrderBarrier();
			}

			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_USE", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_USE"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_STOCK", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_STOCK"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT"));			
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CSS", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_CSS"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"));			

			if (empty(Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS")))
			{
				Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CSS", 
				".polarislink-selecteur li {
	float: left;
	font-size: 8pt;
	margin-right: 4px;
}

.polarislink-critselectblock .polarislink-existeen .lbl, .polarislink-critselectblock .polarislink-icien .vle {
	font-weight: bold;
}

.polarislink-selecteur-end {
	clear: both;
}

.polarislink-selecteur .active {
	background: darkorange;
}

.polarislink-selecteur .active a {
	border: 1px solid darkorange;
}

.polarislink-selecteur .active img {
	opacity: 0.7;
}

.polarislink-selecteur a {
	display: block;
	border: 1px solid #ccc;
	border-radius: 24px;
	width: 48px;
	height: 48px;
	line-height: 24px;
	overflow: hidden;
	text-align: center;
	background: white;
	text-decoration: none;
}

.polarislink-selecteur a:hover,
.polarislink-selecteur a:focus
{
	border: 1px solid darkorange;
	text-decoration: none; 
	outline:none;
}
");
			}
		
			// Associations ?		
			$stores_map = [];
			foreach ($stores as $store)
			{
				$key = Tools::getValue("POLARISLINK_STORESMAP_" . $store["id_store"]);
				if ($key != '')
					$stores_map["code_".$key] = $store["id_store"];
			}
			Configuration::updateValue("POLARISLINK_STORESMAP", json_encode($stores_map));	

			$index = 0;
			do
			{
				$index++;
				$linkId = intval(Tools::getValue('PPL_VEGA_'.$index.'_LINKID'));
				if (!empty($linkId))
				{
					// on retrouve la bonne configuration ...
					$cfg = null;
					foreach ($this->Config as $sk_cfg)
					{
						if ($sk_cfg->LinkID == $linkId)
						{
							$cfg = $sk_cfg;
							break;
						}
					}
					
					if ($cfg == null)
						continue;					
										
					// Ok, étape 1 : on sauve ce qui doit l'être
					$baseCfgName = "PPL_VEGA_".$index."_";
					if (intval(Tools::getValue($baseCfgName.'DELETE')) > 0)
					{
						// Suppression lien
						self::Log(LOG_INFO, "Chg. config. : suppression lien", $cfg->LinkID);						
						$cfg->delete();
						$reload = true;
					}
					else
					{
						$value = strval(Tools::getValue($baseCfgName.'ACCESSNAME'));
						if (!$value || empty($value) || !Validate::isGenericName($value))
							$output .= $this->displayError($this->l("Le nom spécifié n'est pas valide"));
						else
							$cfg->change("LinkName", $value);
						
						$value = strval(Tools::getValue($baseCfgName.'ACCESSKEY'));
						// Génération automatique ?
						if (!$value || empty($value) || !Validate::isGenericName($value) || strlen($value) < 32)
							$output .= $this->displayError($this->l("La clé fournie n'est pas valide : elle doit faire au moins 32 caractères !"));
						else
							$cfg->change("AccessKey", $value);

						// base64 pour cette valeur pour éviter qu'elle soit en clair dans la base et transformée à la volée à la mise en préprod automatique
						$value = strval(Tools::getValue($baseCfgName.'URL_PRODUCTION'));
						$value = empty($value) ? "" : base64_encode($value);
						$cfg->change("ProductionURL", $value);
							
						$cfg->change("AccessActive", intval(Tools::getValue($baseCfgName.'ACCESSOK')));
						$cfg->change("OPT_SIZE_GRID", intval(Tools::getValue($baseCfgName.'SIZE_GRID')));
						
						$cfg->change("LangID", intval(Tools::getValue($baseCfgName.'LANG_ID')));
						$cfg->change("CurrencyID", intval(Tools::getValue($baseCfgName.'CURRENCY_ID')));

						$cfg->change("StockMini", intval(Tools::getValue($baseCfgName.'STOCK_MINI_VALUE')));
						$cfg->change("DefaultWeigth", intval(Tools::getValue($baseCfgName.'WEIGTH_DEFAULT_VALUE')));
						
						$cfg->change("CustomersGroupID", intval(Tools::getValue($baseCfgName.'CUSTOMERSGROUP_ID')));

						$cfg->change("VisibilityWhenSoldOut", Tools::getValue($baseCfgName.'VisibilityWhenSoldOut'));
						

						if (intval(Tools::getValue($baseCfgName.'SUPPLIER_ID')) == intval(Tools::getValue($baseCfgName.'BKSUPPLIER_ID')))
						{
							$output .= $this->displayError($this->l("Le fournisseur associé et le fournisseur de récupération ne peuvent pas être identiques !"));
						}
						else
						{
							$cfg->change("SupplierID", intval(Tools::getValue($baseCfgName.'SUPPLIER_ID')));
							$cfg->change("SupplierBackupID", intval(Tools::getValue($baseCfgName.'BKSUPPLIER_ID')));
						}
						
						$cfg->change("CategoryForRFS", intval(Tools::getValue($baseCfgName.'RFS')));
						$cfg->change("CategoryForManufacturer", intval(Tools::getValue($baseCfgName.'MANUFACTURER')));																	

						foreach ($categoriesOrFeatures as $d)
						{
							$cfg->change("CategoryFor".ucfirst(strtolower($d["k"])), intval(Tools::getValue($baseCfgName.$d["k"])));
							$cfg->change("FeatureFor".ucFirst(strtolower($d["k"])), intval(Tools::getValue($baseCfgName.'F'.$d["k"])));
						}
						
						$cfg->change("CategoryForTags", intval(Tools::getValue($baseCfgName.'TAGS')));			
												
						$cfg->change("DefaultCategoryRoot", intval(Tools::getValue($baseCfgName.'DEFAULT')));			
						$cfg->change("AutoCategory", intval(Tools::getValue($baseCfgName.'AUTOCATEGORY')));
						$cfg->change("SortingCategoryID", intval(Tools::getValue($baseCfgName.'SORTINGCATEGORYID')));
						
						for ($i = 1; $i <= 4; $i++)
							$cfg->change("UseLevel".$i."ForSize", intval(Tools::getValue($baseCfgName.'USEINSIZE_LV'.$i)));
						
						$cfg->change("AttrGroupForAttr0", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV0')));
						$cfg->change("AttrGroupForAttr1", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV1')));
						$cfg->change("AttrGroupForAttr2", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV2')));
						$cfg->change("AttrGroupForAttr3", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV3')));
						$cfg->change("AttrGroupForAttr4", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV4')));
						
						$cfg->change("AttrGroupForAttrG1", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG1')));
						$cfg->change("AttrGroupForAttrG2", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG2')));
						$cfg->change("AttrGroupForAttrG3", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG3')));
						$cfg->change("AttrGroupForAttrG4", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG4')));

						$cfg->change("FeatureForAttr1", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV1')));
						$cfg->change("FeatureForAttr2", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV2')));
						$cfg->change("FeatureForAttr3", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV3')));
						$cfg->change("FeatureForAttr4", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV4')));
						
						$cfg->change("FeatureForAttrG1", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG1')));
						$cfg->change("FeatureForAttrG2", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG2')));
						$cfg->change("FeatureForAttrG3", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG3')));
						$cfg->change("FeatureForAttrG4", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG4')));

						$cfg->change("MethodTags", intval(Tools::getValue($baseCfgName.'TAGS_METHOD')));

						$cfg->change("SlugBuildMethod", Tools::getValue($baseCfgName.'SLUG_BUILD_METHOD'));

						$cfg->change("OrderExportStateID", $this->getMultipleParams($baseCfgName.'EXPORT_SALES_STATUSES', OrderState::getOrderStates($id_lang), 'id_order_state'));
						$cfg->change("CatalogShopID", Tools::getValue($baseCfgName.'CATALOG_SHOP_ID'));
						$cfg->change("PricesShopID", Tools::getValue($baseCfgName.'PRICES_SHOP_ID'));
						
						foreach ($opts as $opt_name => $opt)
							$cfg->change($opt_name, intval(Tools::getValue($baseCfgName.''.$opt_name)));
							
						$cfg->save();
					}

				}
			} while (!empty($linkId));

			// Enfin, les actions ...
			switch (intval(Tools::getValue('PPL_LINK_ACTION')))
			{
				case 1:
					// Nouveau lien
					$cfg = new PolarisLinkSite();
					$cfg->save();
					self::Log(LOG_INFO, "Chg. config. : nouveau lien", $cfg->LinkID);
					$this->Config[] = $cfg;
					break;
			}
			
			if ($reload)
				$this->LoadConfig(false);

			$this->saveHtAccess();
				
			if (empty($output))
				$output .= $this->displayConfirmation($this->l('Vos paramètres ont tous été mis à jour avec succès'));
		}
		
		$list_attrs = array(
							"query" => AttributeGroup::getAttributesGroups($id_lang),
							"id" => "id_attribute_group",
							"name" => "name"
						);

		$list_nullable_attrs = array(
			"query" => [["id_attribute_group" => 0, "name" => "— Ne pas utiliser —"]],
			"id" => "id_attribute_group",
			"name" => "name"
		);

		foreach ($list_attrs["query"] as $v)
			$list_nullable_attrs["query"][] = $v;		
		
		$list_categories = array(
							"query" => Category::getAllCategoriesName(null, false, false),
							"id" => "id_category",
							"name" => "name"
						); 	

		for ($i = 0; $i < count($list_categories["query"]); $i++)
			$list_categories["query"][$i]["name"] = $list_categories["query"][$i]["id_category"]." - ".$list_categories["query"][$i]["name"];
			

		$list_nullable_categories = array(
			"query" => [["id_category" => 0, "name" => "— Ne pas utiliser —"]],
			"id" => "id_category",
			"name" => "name"
		);		

		foreach ($list_categories["query"] as $v)
			$list_nullable_categories["query"][] = $v;				

		$list_features = array(
			"query" => Feature::getFeatures($id_lang),
			"id" => "id_feature",
			"name" => "name"
		); 	

		$list_nullable_features = array(
			"query" => [["id_feature" => 0, "name" => "— Ne pas utiliser —"]],
			"id" => "id_feature",
			"name" => "name"
		);		

		foreach ($list_features["query"] as $v)
			$list_nullable_features["query"][] = $v;		

		$helper = new HelperForm();
			 
		// Module, token and currentIndex
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		 
		// Language
		$helper->default_form_language = $id_lang;
		$helper->allow_employee_form_lang = $id_lang;
		 
		// Title and toolbar
		$helper->title = $this->displayName;
		$helper->show_toolbar = false;        // false -> remove toolbar
		$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper->submit_action = 'submit'.$this->name;
		
		$helper->fields_value['PPL_LINK_ACTION'] = '0';

		$buttons = array(
			'doc-link' => array(
				'title' => "Documentation",
				'name' => 'help',
				'type' => 'button',
				'class' => 'btn btn-default pull-left',
				'icon' => 'process-icon-help',										
				'js' => "window.location='../modules/polarislink/views/Module_Prestashop_PolarisLink.pdf'"
			  )
		 );

		/*$buttons['add-link'] = array(
						'title' => "Ajouter un lien",
						'name' => 'addlink',
						'type' => 'button',
						'class' => 'btn btn-default pull-left expert',
						'icon' => 'process-icon-plus',										
						'js' => "$('#PPL_LINK_ACTION').val(1); $('#configuration_form').submit();"
					 );*/

		$buttons['changelog-link'] = array(
						'title' => "Changelog",
						'name' => 'changelog',
						'type' => 'button',
						'class' => 'btn btn-default pull-left',
						'icon' => 'process-icon-envelope',
						'js' => "window.open('../modules/polarislink/CHANGELOG.html');"
					 );

		$buttons['archive'] = array(
			'title' => "Archives",
			'name' => 'archives',
			'type' => 'button',
			'class' => 'btn btn-default pull-left mr-2',
			'icon' => 'process-icon-preview',
			'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkArchive')."'"
		);

		$buttons['nettoyage'] = array(
			'title' => "Nettoyer journaux",
			'name' => 'cleanlogs',
			'type' => 'button',
			'class' => 'btn btn-default pull-left mr-2',
			'icon' => 'process-icon-delete',
			'js' => "if (confirm('Êtes-vous sûr de vouloir supprimer les entrées de modifications PolarisLink des journaux ?')) window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&tool=clean-logs'"
		);

		$buttons['clear_errors'] = array(
			'title' => "Nettoyer erreurs",
			'name' => 'cleanerrors',
			'type' => 'button',
			'class' => 'btn btn-default pull-left mr-2 expert',
			'icon' => 'process-icon-delete',
			'js' => "if (confirm('Êtes-vous sûr de vouloir supprimer les erreurs de PolarisLink des journaux ?')) window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&tool=clean-errors'"
		);

		if ($isVega)
			$buttons['gotoapi'] = array(
				'title' => "Test API",
				'name' => 'test_api',
				'type' => 'button',
				'class' => 'btn btn-default pull-left mr-2 expert',
				'icon' => 'process-icon-next',
				'js' => "window.open('../?fc=module&module=polarislink&controller=api_version')"
			);
		
		$buttons['expert'] = array(
				'title' => "Options expert",
				'name' => 'expert',
				'type' => 'button',
				'class' => 'btn btn-default pull-left mr-2',
				'icon' => 'process-icon-modules-list',
				'js' => "$('.expert').toggle()"
			);
		
		$buttons['upgrade'] = array(
									'title' => "Chercher les mises à jour",
									'name' => 'upgrade',
									'type' => 'button',
									'class' => 'btn btn-default pull-left mr-2',
									'icon' => 'process-icon-update',
									'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkVegaAutoUpdater')."'"
							  );			

		$gblopts = array(
			array('type' => 'hidden', 'name' => 'PPL_LINK_ACTION'),
			array(
				'type' => 'html',
				'name' => 'html_lbl',
				'label' => $this->l("Version du module"),
				'html_content' => "<h4>".PolarisLink::GetFullVersion()."</h4>"),
			
			array(
				'type' => 'switch',
				'label' => $this->l("Permettre la connexion de VEGA Informatique à votre Backoffice Prestashop"),
				'name' => "POLARISLINK_ALLOW_MAINTENANCE_ACCESS",
				'class' => 't', 'is_bool' => true,
				'values' => array(
								array('id' => 'active_on', 'value' => 1),
								array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
							)
			),

			array(
				'type' => 'switch',
				'form_group_class' => 'expert',
				'label' => $this->l("Nettoyage automatique des tables de statistiques"),
				'hint' => $this->l("Les tables de statistiques peuvent devenir rapidement volumineuses et ne servent pas si vous disposez d'un analyseur de trafic (Google Analytics, Matomo). Activez cette options pour ne garder que les données sur un mois."),
				'name' => "POLARISLINK_AUTOCLEAN_STATS",
				'class' => 't', 
				'is_bool' => true,
				'values' => array(
								array('id' => 'active_on', 'value' => 1),
								array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
							)
			),

			array(
				'type' => 'switch',
				'label' => $this->l("Conversion automatique des JPEG en WEBP"),
				'hint' => $this->l("Servir des *.webp en lieu et place des *.jpeg quand le navigateur de l'internaute le permet"),
				'name' => "POLARISLINK_USE_WEBP",
				'class' => 't', 
				'is_bool' => true,
				'values' => array(
								array('id' => 'active_on', 'value' => 1),
								array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
							)
			),

			array(
				'type' => 'text',
				'form_group_class' => 'expert',
				'label' => $this->l("Ne pas exporter les commandes avec un ID strictement inférieur à :"),
				'name' => "POLARISLINK_ORDER_BARRIER",
			),			
		);

		if ($isVega)
		{
			$gblopts[] = array(
				'type' => 'text',
				'form_group_class' => 'expert',
				'label' => $this->l("Codes de suivi GoogleAnalytics"),
				'hint' => 'Un seul tag par site !',
				'name' => 'POLARISLINK_GOOGLE_TAGS',
			);
		}

		// Init Fields form array		
		$fields_form[0]['form'] = array(
			'input' => $gblopts,
			'buttons' => $buttons,
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'btn btn-default pull-right'
			)
		);	

		$fields_form[0]['form']['input'][] = array(
												'type' => 'switch',
												'label' => $this->l("Mise à jour des données par tâche CRON"),
												'hint' => $this->l("Met à jour les données par tâche CRON plutôt que par appel PHP externe. Cette méthode nécessite de pouvoir accéder à votre serveur d'hébergement."),
												'name' => "POLARISLINK_CRON_DATAUPDATE",
												'class' => 't', 'is_bool' => true,
												'values' => array(
																array('id' => 'active_on', 'value' => 1),
																array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
															)
												);

		if (Configuration::get("POLARISLINK_CRON_DATAUPDATE"))
		{
			$fields_form[0]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<h4>Mise à jour des données par tâche CRON</h4>
									<p>
									Éditez votre configuration CRON (<code>crontab -e</code>) pour ajouter la ligne suivante :									
<pre>* * * * * /usr/bin/php "._PS_MODULE_DIR_."polarislink/utils/run_jobs.php --cfg "._PS_CONFIG_DIR_."
</pre></p><p><sup>NOTES : </sup> Si vous avez plusieurs versions PHP ou si votre environnement est en <i>root-jail</i> ou <i>root-cage</i>, assurez-vous que les chemins sont corrects.</p>",
				);				
		}
		
		$configPage = '';
		$cfg_index = 0;
		
		foreach ($this->Config as $cfg)
		{
			$cfg_index++;
			
			// Test d'une des clés valides
			$result401 = null;

			$categoriesOrFeatures = [
				[ "t" => $cfg->CategoryName1, "k" => "CATEGORY1" ],
				[ "t" => $cfg->CategoryName2, "k" => "CATEGORY2" ],
				[ "t" => $cfg->CategoryName3, "k" => "CATEGORY3" ],
				[ "t" => $cfg->CategoryName4, "k" => "CATEGORY4" ],
				[ "t" => "saisons Polaris", "k" => "SEASON" ],
				[ "t" => "collections Polaris", "k" => "COLLECTION" ],						
			];

			// Dans le cas de CGI ou PHP-FPM, on vérifie que le paramétrage est correct !
			if (strpos(php_sapi_name(), "cgi") || strpos(php_sapi_name(), "fpm"))
			{
				if (Configuration::Get("PS_WEBSERVICE_CGI_HOST") == 0)
				{
					if ($cfg->AccessActive && !$this->test401($cfg->AccessKey))
					{
						$lnk = new Link();
						$result401 = array(
							'type' => 'html',
							'name' => 'html_401',
							'style' => 'color: red;',
							'label' => '',
							'html_content' => '<div style="color: crimson; border: 3px solid crimson; padding: 8px;"><strong>&#9888; ATTENTION : le test de la clé a échoué (erreur 401) !</strong><br/>'.
												"Le module a détecté que vous utilisiez PHP-FPM ou PHP en mode CGI et vous n'avez pas activé le mode CGI de Prestashop. L'erreur provient peut-être de là. Pour activer le mode CGI : ".												
												"<ol>".												
												"<li><strong>Activer le mode CGI de PHP</strong> sur la page «&nbsp;<a target=\"_blank\" href=\"".$this->context->link->getAdminLink('AdminWebservice')."\">Paramètres avançés &gt; Webservice</a>&nbsp;» ; n'oubliez pas d'enregistrer !</li>".
												"<li><strong>Regénérer le .htaccess</strong> sur la page «&nbsp;<a target=\"_blank\" href=\"".$this->context->link->getAdminLink('AdminParentMeta')."\">Paramètres de la boutique &gt; Trafic &amp; SEO</a>&nbsp;» puis cliquer sur le bouton '<em>Enregistrer</em>' des URL de la boutique.</li>".
												"</ol>".
												'</div>'
						);
					}
				}
			}			

			$baseCfg = 'PPL_VEGA_'.$cfg_index.'_';
			
			$buttons = array();
			if ($isVega)
			{
				$url_production = empty($cfg->ProductionURL) ? "" : base64_decode($cfg->ProductionURL);
				$url_production = substr($url_production, strpos($url_production, "//"));
				$url_current = substr(_PS_BASE_URL_.__PS_BASE_URI__, strpos(_PS_BASE_URL_.__PS_BASE_URI__, "//"));

				if (!empty($url_production) && $url_production != $url_current)
				{
					$buttons[] = array(
						'title' => "Synchro depuis prod",
						'class' => 'btn-warning pull-left',
						'icon' => 'process-icon-expand',
						'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkSynchroFromProd')."&id_link=".$cfg->LinkID."'"
					);
				}

				if (count($this->Config) > 1)
					$buttons[] = array(
						'title' => "Supprimer",
						'class' => 'btn-danger pull-left',
						'icon' => 'process-icon-delete',
						'js' => "if (confirm('Êtes-vous sûr de vouloir supprimer le lien ".$cfg->LinkName." ?')) { $('#".$baseCfg."DELETE').val(1); $('#configuration_form').submit(); } ",
					);
			}

			$buttons[] = array(
					'title' => "Mappage catégories",
					'class' => 'btn pull-right',
					'icon' => 'process-icon-edit',
					'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkEditCategoryMappage')."&id_link=".$cfg->LinkID."'"
				);

			$buttons[] = array(
					'title' => "Tri des tailles",
					'class' => 'btn pull-right',
					'icon' => 'process-icon-expand',
					'js' => "if (confirm('Êtes-vous sûr de vouloir retrier les tailles du site ".$cfg->LinkName." ?')) { window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=sort-sizes' } "
			);

			$buttons[] = array(
					'title' => $cfg->Resync > 0 ? "Resynchronisation demandée" : "Resynchroniser",
					'class' => 'btn pull-right'.($cfg->Resync > 0 ? " btn-warning" : ""),
					'icon' => 'process-icon-refresh',
					'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=resync'"
			);

			$buttons[] = array(
					'title' => $cfg->ReExportSales > 0 ? "Réexportation des ventes demandée" : "Réexporter les ventes",
					'class' => 'btn pull-right'.($cfg->ReExportSales > 0 ? " btn-warning" : ""),
					'icon' => 'process-icon-refresh',
					'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=reexport_sales'"
			);

			$buttons[] = array(
					'title' => $cfg->ReExportCustomers > 0 ? "Réexportation des clients demandée" : "Réexporter les clients",
					'class' => 'btn pull-right'.($cfg->ReExportCustomers > 0 ? " btn-warning" : ""),
					'icon' => 'process-icon-refresh',
					'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=reexport_customers'"
			);
			
			// Init Fields form array		
			$fields_form[$cfg_index]['form'] = array(
				'input' => array(
					array(
						'type' => 'html',
						'name' => 'html_lbl',						
						'html_content' => "<div style=\"text-align: right;\">[&nbsp;<a href=\"#bottom\">Aller aux utilitaires</a>&nbsp;]</div>"
					),

					array(
						'type' => 'hidden',
						'name' => $baseCfg.'LINKID'
					),
					array(
						'type' => 'hidden',
						'name' => $baseCfg.'DELETE'
					),					
					array(
						'type' => 'html',
						'name' => 'html_lbl',
						'html_content' => "<h4>Accès</h4>"
					),					

					array(
						'type' => 'text',
						'label' => $this->l("Nom"),
						'hint' => $this->l("Ce nom est purement indicatif. Il sert simplement à reconnaître rapidement ce paramétrage."),
						'name' => $baseCfg.'ACCESSNAME',					
						'size' => 40,
						'required' => true,
					),
					array(
						'type' => 'textbutton',
						'label' => $this->l("Clé d'accès"),
						'hint' => $this->l("Cette clé d'accès est à reporter dans la configuration de votre site dans Polaris. Elle permet d'établir la liaison entre les deux systèmes de manière fiable et sécurisée."),
						'name' => $baseCfg.'ACCESSKEY',	
						'id' => 'code_'.$cfg_index,
						'size' => 128,
						'required' => true,
						'button' => array(
							'label' => $this->l('Générer une nouvelle clé'),
							'attributes' => array(
								'onclick' => 'VEGAGenKey(\'code_'.$cfg_index.'\', 32)'
								)
							)
					),
					$isVega ? array(
						'type' => 'text',
						'label' => $this->l("URL de production"),
						'hint' => $this->l("Indiquez ici l'URL de production quand il s'agit d'un environnement de préproduction"),
						'name' => $baseCfg.'URL_PRODUCTION',	
						'size' => 128,
						'required' => false
					) : null,
					array(
						'type' => 'switch',
						'label' => $this->l("Accès actif"),
						'hint' => $this->l("Utilisez cette option pour activer ou désactiver l'accès sans pour autant perdre votre clé."),
						'name' => $baseCfg.'ACCESSOK',
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1, 'label' => $this->l('Activé')),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					),
					$result401,
					array(
						'type' => 'html',
						'name' => 'html_polaris',
						'label' => 'Polaris détecté',
						'html_content' => "<strong>".$cfg->PolarisLicence."</strong><br/>version ".$cfg->PolarisVersion
					),

					array(
						'type' => 'html',
						'name' => 'html_lbl',
						'html_content' => "<hr><h4>Options générales</h4>"
					),
					array(
						'type' => 'select',
						'label' => $this->l("Langue"),
						'name' => $baseCfg.'LANG_ID',
						'required' => true,
						'options' => array(
										'query' => Language::getLanguages(),
										'id' => 'id_lang',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Devise gérée"),
						'name' => $baseCfg.'CURRENCY_ID',
						'required' => true,
						'options' => array(
										'query' => Currency::getCurrencies(),
										'id' => 'id_currency',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Gérer les produits du site"),
						'name' => $baseCfg.'CATALOG_SHOP_ID',
						"default" => "",
						'options' => array(
										'query' => $shops,
										'id' => 'id_shop',
										'name' => 'name',
										),
						'required' => true,
					),
					array(
						'type' => 'select',
						'label' => $this->l("Gérer les prix du site"),
						'name' => $baseCfg.'PRICES_SHOP_ID',
						"default" => "",
						'options' => array(
										'query' => $shops,
										'id' => 'id_shop',
										'name' => 'name',
										),
						'required' => true,
					),
					array(
						'type' => 'text',
						
						'label' => $this->l("Stock minimum de sécurité"),
						'name' => $baseCfg.'STOCK_MINI_VALUE',
						"default" => "0",						
						'required' => true,
						'class' => 'input fixed-width-md',
						'suffix' => 'pièce(s)',
						'hint' => "Cette option n'est valide que si vous n'utilisez pas le stock avançé, elle permet de retrancher du stock envoyé de Polaris une quantité de sécurité pour réserver les dernières pièces aux boutiques.",
					),
					array(
						'type' => 'text',
						
						'label' => $this->l("Poids des produits par défaut"),
						'name' => $baseCfg.'WEIGTH_DEFAULT_VALUE',
						"default" => "0",						
						'required' => true,
						'class' => 'input fixed-width-md',
						'suffix' => 'g',
						'hint' => "Ce poids est utilisé lorsque Polaris communique un poids d'article égal ou inférieur à 0.",
					),
					array(
						'type' => 'select',
						'label' => $this->l("Groupe de clients"),
						'hint' => $this->l("Groupe de clients dans lequel enregistrer les clients provenant de l'importation depuis Polaris"),
						'name' => $baseCfg.'CUSTOMERSGROUP_ID',
						'required' => true,
						'options' => array(
										'query' => Group::getGroups($id_lang),
										'id' => 'id_group',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'form_group_class' => 'expert',
						'label' => $this->l("Fournisseur associé"),
						'hint' => $this->l("Tous les produits associés à ce fournisseur seront gérés par la passerelle."),
						'name' => $baseCfg.'SUPPLIER_ID',
						'required' => true,
						'options' => array(
										'query' => $suppliers,
										'id' => 'id_supplier',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'form_group_class' => 'expert',
						'label' => $this->l("Fournisseur de récupération"),
						'hint' => $this->l("Tous les produits associés à ce fournisseur seront gérés par la passerelle, mais seulement leur stock et leur prix (achat et vente)."),
						'name' => $baseCfg.'BKSUPPLIER_ID',
						'required' => true,
						'options' => array(
										'query' => $suppliers,
										'id' => 'id_supplier',
										'name' => 'name',
										)
					),
				),	
		
				'buttons' => $buttons
			);						

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Classement des références</h4>"
			);

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'text',
				'label' => $this->l("Méthode de construction des slugs/liens produits"),
				'hint' => $this->l("Utilisez {reference}, {name}, {niveau1} à {niveau4}, {c1} à {c4}, {season}, {manufacturer}, {collection}"),
				'name' => $baseCfg.'SLUG_BUILD_METHOD',
				"default" => "",
				'required' => true,
			);

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'select',
				'label' => $this->l("Afficher les produits en rupture de stock"),
				'desc' => $this->l("Cette option permet de paramétrer la visibilité des produits automatiquement quand ils tombent en rupture de stock, et à \"Partout\" quand ils sont réapprovisionnés"),
				'name' => $baseCfg.'VisibilityWhenSoldOut',
				"default" => "",
				"options" => $list_visibilities,
				"required" => true,						
			);	


			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'label' => $this->l("Classement automatique"),
						'hint' => $this->l("Utilisez cette option pour classer votre produit selon un type de catégorie de Polaris."),
						'name' => $baseCfg.'AUTOCATEGORY',
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1, 'label' => $this->l('Activé')),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					);	
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie de classement"),
						'desc' => $this->l("Précise dans quelle catégorie de Prestashop sont rangés les produits à classer quand vous n'utilisez pas le classement automatique."),
						'name' => $baseCfg.'SORTINGCATEGORYID',
						"default" => "",
						"options" => $list_categories,
						"required" => true,						
					);	
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Racine de la catégorie par défaut"),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop est sélectionnée la catégorie par défaut du produit. Si vous sélectionnez la racine, la catégorie par défaut ne sera pas modifiée."),
						'name' => $baseCfg.'DEFAULT',
						"default" => "",
						"options" => $list_categories,
						'required' => true,						
					);					
			
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie racine pour les RFS"),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les RFS de Polaris"),
						'name' => $baseCfg.'RFS',
						'form_group_class' => 'expert',
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);

			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie racine des marques"),
						'desc' => $this->l("Pour un automappage par marque, précisez la catégorie pour utiliser la marque ici"),
						'name' => $baseCfg.'MANUFACTURER',
						'form_group_class' => 'expert',
						"default" => "",
						"options" => $list_nullable_categories,
						'required' => false,
					);					

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => '<hr><h4>Configuration des thèmes Polaris</h4>'
			);

			$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'radio',
					'label' => $this->l("Méthode de traitement des thèmes Polaris"),
					'hint' => $this->l("Précise la méthode de gestion des multi-thèmes Polaris"),
					'name' => $baseCfg.'TAGS_METHOD',
					"default" => "",
					"values" => array(
									array("id" => 0, "value" => 0, "label" => "Ignorer les thèmes"),
									array("id" => 1, "value" => 1, "label" => "Considérer comme des catégories"),
									array("id" => 2, "value" => 2, "label" => "Considérer comme des caractéristiques"),
									array("id" => 3, "value" => 3, "label" => "Considérer comme des mots-clés"),
								),
					'required' => true,
				);

			$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'form_group_class' => 'expert',
					'label' => $this->l("Catégorie racine pour les mots-clés"),
					'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les mots clés de Polaris lorsqu'ils sont utilisés en mode catégorie"),
					'name' => $baseCfg.'TAGS',
					"default" => "",
					"options" => $list_categories,
					'required' => true,
				);

			foreach ($categoriesOrFeatures as $d)
			{
				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'html',
					'name' => 'html_lbl',
					'html_content' => '<hr><h4>Configuration des '.$d["t"].'</h4>'
				);

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'form_group_class' => 'expert',
						'label' => $this->l("Catégorie racine pour les ".$d["t"]),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les ".$d["t"]),
						'name' => $baseCfg.$d["k"],
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Caractéristique pour les ".$d["t"]),
						'desc' => $this->l("Si vous souhaitez distribuer cette donnée dans les caractéristiques, précise à laquelle attribuer les ".$d["t"]),
						'name' => $baseCfg.'F'.$d["k"],
						"default" => "",
						"options" => $list_nullable_features,
						'required' => false,
					);
			}

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Configuration des tailles</h4>"
			);
			
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Groupe d'attributs pour la taille"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de taille doivent être rangées"),					
						'name' => $baseCfg.'ATTR_FOR_LV0',
						"default" => "",
						"options" => $list_attrs,
						'required' => true
					);

			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'label' => $this->l("Gérer les grilles de tailles"),
						'hint' => "Quand cette option est activée et si un groupe d'attributs du même nom que la grille de tailles Polaris existe sous Prestashop, la passerelle gérera la taille dans le groupe d\'attributs correspondant; dans le cas contraire, les tailles seront gérées dans le groupe d'attributs précisé ci-dessus.",
						'name' => $baseCfg."SIZE_GRID",
						'class' => 't', 'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
						);
										
			for ($i = 1; $i <= 4; $i++)
			{
				$propName = "LevelName".$i;

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'html',
					'name' => 'html_lbl',
					'html_content' => '<hr><h4>Configuration du niveau Polaris n°'.$i.' : <strong>'.$cfg->$propName.'</strong> </h4>'
				);

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'switch',
					'label' => $this->l("Utiliser ce niveau comme complément de taille"),
					'name' => $baseCfg.'USEINSIZE_LV'.$i,
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0),
								)
					);

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Groupe d'attributs"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau ".$i." sous Polaris doivent être gérée sur Prestashop"),					
						'name' => $baseCfg.'ATTR_FOR_LV'.$i,
						"default" => "",
						"options" => $list_nullable_attrs,
						'required' => false,
					);				

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Caractéristique"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
						'name' => $baseCfg.'FEATURE_FOR_LV'.$i,
						"default" => "",
						"options" => $list_nullable_features,
						'required' => false,
					);

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Groupe d'attributs pour le niveau générique"),
					'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
					'name' => $baseCfg.'ATTR_FOR_LVG'.$i,
					"default" => "",
					"options" => $list_nullable_attrs,
					'required' => false,
				);				

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Caractéristique pour le niveau générique"),
					'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
					'name' => $baseCfg.'FEATURE_FOR_LVG'.$i,
					"default" => "",
					"options" => $list_nullable_features,
					'required' => false,
				);
			}
			
			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Options d'import/export</h4>"
			);
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'checkbox',
						'label' => $this->l("Exporter les commandes ayant obtenu au moins une fois le statut"),
						'name' => $baseCfg.'EXPORT_SALES_STATUSES',
						"default" => "",
						'values' => array(
										'query' => OrderState::getOrderStates($id_lang),
										'id' => 'id_order_state',
										'name' => 'name',
										),
						'required' => true,
					);
						
			foreach ($opts as $opt_name => $opt)
			{
				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'form_group_class' => (isset($opt['e']) && $opt['e']) ? 'expert' : '',
						'label' => $this->l($opt['t']),
						'hint' => $this->l($opt['d']),
						'name' => $baseCfg.''.$opt_name,
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					);
			}			
			
			// Load current value
			if (!Configuration::hasKey("POLARISLINK_ALLOW_MAINTENANCE_ACCESS"))
				Configuration::updateValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS", 1);
			$helper->fields_value["POLARISLINK_ALLOW_MAINTENANCE_ACCESS"] = Configuration::get("POLARISLINK_ALLOW_MAINTENANCE_ACCESS");			

			if (!Configuration::hasKey("POLARISLINK_CRON_DATAUPDATE"))
				Configuration::updateValue("POLARISLINK_CRON_DATAUPDATE", 0);

			$helper->fields_value["POLARISLINK_CRON_DATAUPDATE"] = Configuration::get("POLARISLINK_CRON_DATAUPDATE");
			$helper->fields_value["POLARISLINK_AUTOCLEAN_STATS"] = Configuration::get("POLARISLINK_AUTOCLEAN_STATS");
			$helper->fields_value["POLARISLINK_USE_WEBP"] = Configuration::get("POLARISLINK_USE_WEBP");
			if ($isVega)
				$helper->fields_value["POLARISLINK_GOOGLE_TAGS"] = Configuration::Get("POLARISLINK_GOOGLE_TAGS", "");

			$helper->fields_value["POLARISLINK_ORDER_BARRIER"] = $orderBarrier;

			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_USE"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_USE");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_STOCK"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_STOCK");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_CSS"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS");			
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL");						

			$storesMap = json_decode(Configuration::get("POLARISLINK_STORESMAP"), true);
			if (!empty($storesMap))
			{
				foreach ($storesMap as $refPolaris => $refPresta)
					$helper->fields_value["POLARISLINK_STORESMAP_".$refPresta] = substr($refPolaris, 5);
			}

			$helper->fields_value[$baseCfg.'DELETE'] = "0";
			$helper->fields_value[$baseCfg.'LINKID'] = $cfg->LinkID;
			$helper->fields_value[$baseCfg.'ACCESSNAME'] = $cfg->LinkName;
			$helper->fields_value[$baseCfg.'ACCESSKEY'] = $cfg->AccessKey;
			$helper->fields_value[$baseCfg.'ACCESSOK'] = $cfg->AccessActive;

			$helper->fields_value[$baseCfg.'URL_PRODUCTION'] = empty($cfg->ProductionURL) ? "" : base64_decode($cfg->ProductionURL);
			
			$helper->fields_value[$baseCfg.'LANG_ID'] = $cfg->LangID;
			$helper->fields_value[$baseCfg.'CURRENCY_ID'] = $cfg->CurrencyID;
			$helper->fields_value[$baseCfg.'STOCK_MINI_VALUE'] = $cfg->StockMini;
			$helper->fields_value[$baseCfg.'WEIGTH_DEFAULT_VALUE'] = $cfg->DefaultWeigth;
			
			$helper->fields_value[$baseCfg.'CATALOG_SHOP_ID'] = $cfg->CatalogShopID;
			$helper->fields_value[$baseCfg.'PRICES_SHOP_ID'] = $cfg->PricesShopID;
			
			$helper->fields_value[$baseCfg.'CUSTOMERSGROUP_ID'] = $cfg->CustomersGroupID;
			$helper->fields_value[$baseCfg.'SUPPLIER_ID'] = $cfg->SupplierID;
			$helper->fields_value[$baseCfg.'BKSUPPLIER_ID'] = $cfg->SupplierBackupID;
			$helper->fields_value[$baseCfg.'TAGS_METHOD'] = $cfg->MethodTags;
			$helper->fields_value[$baseCfg.'SLUG_BUILD_METHOD'] = $cfg->SlugBuildMethod;			
									
			$helper->fields_value[$baseCfg.'RFS'] = $cfg->CategoryForRFS;
			$helper->fields_value[$baseCfg.'MANUFACTURER'] = $cfg->CategoryForManufacturer;

			$helper->fields_value[$baseCfg.'TAGS'] = $cfg->CategoryForTags;

			foreach ($categoriesOrFeatures as $d)
			{
				$k = "CategoryFor".ucfirst(strtolower($d["k"]));
				$helper->fields_value[$baseCfg.$d["k"]] = $cfg->$k;
				$k = "FeatureFor".ucfirst(strtolower($d["k"]));
				$helper->fields_value[$baseCfg.'F'.$d["k"]] = $cfg->$k;
			}
						
			$helper->fields_value[$baseCfg.'AUTOCATEGORY'] = $cfg->AutoCategory;			
			$helper->fields_value[$baseCfg.'SORTINGCATEGORYID'] = $cfg->SortingCategoryID;			
			$helper->fields_value[$baseCfg.'DEFAULT'] = $cfg->DefaultCategoryRoot;			

			$helper->fields_value[$baseCfg.'VisibilityWhenSoldOut'] = $cfg->VisibilityWhenSoldOut;			
			
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV0'] = $cfg->AttrGroupForAttr0;
			$helper->fields_value[$baseCfg.'SIZE_GRID'] = $cfg->OPT_SIZE_GRID;

			$helper->fields_value[$baseCfg.'USEINSIZE_LV1'] = $cfg->UseLevel1ForSize;
			$helper->fields_value[$baseCfg.'USEINSIZE_LV2'] = $cfg->UseLevel2ForSize;
			$helper->fields_value[$baseCfg.'USEINSIZE_LV3'] = $cfg->UseLevel3ForSize;
			$helper->fields_value[$baseCfg.'USEINSIZE_LV4'] = $cfg->UseLevel4ForSize;


			$helper->fields_value[$baseCfg.'ATTR_FOR_LV1'] = $cfg->AttrGroupForAttr1;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV2'] = $cfg->AttrGroupForAttr2;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV3'] = $cfg->AttrGroupForAttr3;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV4'] = $cfg->AttrGroupForAttr4;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG1'] = $cfg->AttrGroupForAttrG1;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG2'] = $cfg->AttrGroupForAttrG2;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG3'] = $cfg->AttrGroupForAttrG3;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG4'] = $cfg->AttrGroupForAttrG4;

			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV1'] = $cfg->FeatureForAttr1;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV2'] = $cfg->FeatureForAttr2;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV3'] = $cfg->FeatureForAttr3;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV4'] = $cfg->FeatureForAttr4;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG1'] = $cfg->FeatureForAttrG1;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG2'] = $cfg->FeatureForAttrG2;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG3'] = $cfg->FeatureForAttrG3;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG4'] = $cfg->FeatureForAttrG4;
			
			foreach ($opts as $opt_name => $opt)
				$helper->fields_value[$baseCfg.''.$opt_name] = $cfg->$opt_name;
		 
			$values = $cfg->OrderExportStateID;
			if ($values != null && count($values) > 0)
			{
				foreach ($values as $state)
					$helper->fields_value[$baseCfg.'EXPORT_SALES_STATUSES_'.$state] = 1;
			}
		 
		}

		// Configuration du sélecteur de produits connexes
		$special_tabs = [];		
		$special_tabs[] = [ "idx" => ++$cfg_index, "title" => "Sélecteur produits" ];		

		// Init Fields form array		
		$fields_form[$cfg_index]['form'] = array(
			'input' => array(
				array(
					'type' => 'html',
					'name' => 'htmllbl',
					'html_content' => '<hr/><h4>Sélecteur de produits connexes</h4>'.												
										'<p>Si vous utilisez le sélecteur, n\'oubliez pas de paramétrer le format d\'image <code>selection</code> pour les produits, avec une taille par défaut de 48px × 48px.</p>'.
										'<p>Le sélecteur sera injecté sous le prix du produit. Si vous désirez le placer ailleurs, désactivez le sélecteur et utilisez le hook suivant dans votre template : <code>{hook h="actionPolarisLinkProductSelector" product_id=$product->id}</code></p>',
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Sélecteur de produits connexes sur les pages produits"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_USE",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),						
				array(
					'type' => 'switch',
					'label' => $this->l("Cacher les déclinaisons sans stock"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_STOCK",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Cacher la déclinaison actuellement affichée"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Gérer les liens canoniques des variantes automatiquement"),
					'hint' => 'Protection contre le <i>near-duplicate content (panda)',
					'desc' => "Si vous utilisez la même description pour toutes les variantes (couleur notamment) d'un même produit, il est possible que Google détecte un contenu presque-dupliqué et que votre ".
								"référencement soit pénalisé. En activant cette option, PolarisLink génèrera – s'il le juge utile – le lien canonique dans l'entête HTTP pour signifier à Google que l'indexation de la page variante n'est pas nécessaire.",
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_CANONICAL",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'textarea',
					'label' => $this->l("Style d'affichage du sélecteur"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_CSS",							
				),
			));
		
		// La configuration des magasins n'est pas encore au programme
		if (false)
		{
			$special_tabs[] = [ "idx" => ++$cfg_index, "title" => "Association Magasins" ];

			$storesAssocs = array(array(
				'type' => 'html',
				'name' => 'htmllbl',
				'html_content' => '<hr/><h4>Association des magasins Polaris/Prestashop</h4>'.												
									'<p>Utilisez ce panneau pour associer les magasins Polaris aux magasins de Prestashop. '.
									'L\'association est utile lorsque vous voulez obtenir une source d\'information pour le stock par boutique.</p>'.
									'<p>En cas de modification de ce tableau, il est vivement conseillé de faire un renvoi total du stock</p>',
				));

			// Ajout des magasins...
			foreach ($stores as $store)
			{
				$storesAssocs[] = array(
					'type' => 'text',				
					'label' => sprintf("<strong>%s</strong>, %s  - Id %d", 
							$store["name"], 
							$store["city"],
							$store["id_store"]),
					'name' => "POLARISLINK_STORESMAP_".$store["id_store"],
					"default" => "",						
					'class' => 'input fixed-width-md',
					'prefix' => 'Code Polaris du magasin',
				);
			}

			// Init Fields form array
			$fields_form[$cfg_index]['form'] = array(
				'input' => $storesAssocs
				);
		}
		
		$generatedForm = $helper->generateForm($fields_form);
		
		// Change fieldset to pane
		$panePage = '<ul class="nav nav-tabs">';
		
		$cfg_index = 0;
		foreach ($this->Config as $cfg)
		{
			$cfg_index++;
			$tabPageName = "fieldset_".$cfg_index."_".$cfg_index;
			
			if ($cfg_index == 1)
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade in active" id="'.$tabPageName.'"', $generatedForm);	
			else
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade" id="'.$tabPageName.'"', $generatedForm);
			
			$panePage .= '<li class="'.(($cfg_index == 1) ? 'active' : '').'"><a data-toggle="tab" href="#'.$tabPageName.'">'.$cfg->LinkName.'</a></li>';
		}

		foreach ($special_tabs as $cfg)
		{
			$cfg_index = $cfg["idx"];
			$tabPageName = "fieldset_".$cfg_index."_".$cfg_index;
			
			if ($cfg_index == 1)
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade in active" id="'.$tabPageName.'"', $generatedForm);	
			else
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade" id="'.$tabPageName.'"', $generatedForm);
			
			$panePage .= '<li class="'.(($cfg_index == 1) ? 'active' : '').'"><a data-toggle="tab" href="#'.$tabPageName.'">'.$cfg["title"].'</a></li>';
		}
		
		$panePage .= '</ul>';		
		$panePage .= '<div class="tab-content">';
		
		//var_dump($generatedForm);
		$generatedForm = str_replace('<div class="panel tab-pane fade in active" id="fieldset_1_1">', $panePage . '<div class="panel tab-pane fade in active" id="fieldset_1_1">', $generatedForm);
		$generatedForm = str_replace('</form>', '</div></form>', $generatedForm);
		
		$generatedForm = str_replace('class="btn btn-default btn-danger', 'class="btn btn-danger', $generatedForm);
		
		$configPage .= $generatedForm;
		$configPage .= '<div id="bottom"></div>';

		$configPage .= "<script>$('.expert').hide();</script>";
		
		$this->sendMetrics();
		
		return $output.$configPage;
	}

	public function loadRelatedProductsInfo($products_id)
	{
		// On regarde si les infos ne seraient pas déjà en cache
		if (count($products_id) == 1 && isset($this->RelatedProductsInfoCache[$products_id[0]]))
			return $this->RelatedProductsInfoCache[$products_id[0]];

		$db = Db::getInstance(_PS_USE_SQL_SLAVE_);

		$products = array();
		$productsDesc = array();
		$productsAttrs = array();
		
		if (count($products_id) > 0)
		{
			foreach ($products_id as $id)
				$products[$id] = array();

			$group_per_product = array();
			$products_per_group = array();
			$cacheProducts = array();
						
			foreach ($db->executeS("SELECT DISTINCT g.id_product, g.id_group FROM "._DB_PREFIX_."polarislink_products_group g LEFT JOIN "._DB_PREFIX_."product AS p ON (p.id_product = g.id_product) WHERE p.id_product IS NOT NULL AND id_group IN (SELECT id_group FROM "._DB_PREFIX_."polarislink_products_group WHERE id_product IN (".join(",", $products_id)."))") as $row)
			{
				if (!isset($cacheProducts[$row["id_product"]]))
					$cacheProducts[$row["id_product"]] = new Product($row["id_product"], true, $this->context->language->id, $this->context->shop->id);
				
				if (!isset($products_per_group[$row["id_group"]]))
					$products_per_group[$row["id_group"]] = array();
								
				$products_per_group[$row["id_group"]][] = $row["id_product"];
				$group_per_product[$row["id_product"]] = $row["id_group"];
			}
			
			foreach ($products_id as $k)
			{
				foreach ($products_per_group[$group_per_product[$k]] as $id_product)
					$products[$k][] = $cacheProducts[$id_product];
			}

			if (count($cacheProducts) > 0)
			{
				foreach ($db->executeS("SELECT DISTINCT pa.id_product, a.id_attribute_group, a.id_attribute, la.name, a.color, p.active ".
										"FROM "._DB_PREFIX_."product_attribute AS pa ".
										"LEFT JOIN "._DB_PREFIX_."product AS p ON (p.id_product = pa.id_product) ".
										"LEFT JOIN "._DB_PREFIX_."product_attribute_combination AS pac ON (pac.id_product_attribute = pa.id_product_attribute) ".
										"LEFT JOIN "._DB_PREFIX_."attribute AS a ON (a.id_attribute = pac.id_attribute) ".
										"LEFT JOIN "._DB_PREFIX_."attribute_group AS ag ON (ag.id_attribute_group = a.id_attribute_group) ".
										"LEFT JOIN "._DB_PREFIX_."attribute_lang AS la ON (la.id_attribute = a.id_attribute AND la.id_lang = ".$this->context->language->id.") ".
										"WHERE pa.id_product IN (".join(",", array_keys($cacheProducts)).") ".
										"ORDER BY ag.position, a.position, pa.id_product") as $row)
				{
					$id_product = $row["id_product"];

					if (!isset($productsDesc[$id_product]))
						$productsDesc[$id_product] = array();
					
					if (!isset($productsDesc[$id_product][$row["id_attribute_group"]]))
						$productsDesc[$id_product][$row["id_attribute_group"]] = array();
					
					if (!isset($productsDesc[$id_product][$row["id_attribute_group"]][$row["id_attribute"]]))
						$productsDesc[$id_product][$row["id_attribute_group"]][$row["id_attribute"]] = array(
																											"id_attribute" => $row["id_attribute"],
																											"name" => $row["name"],
																											"color" => $row["color"],
																										);
					
					if ($row["active"])
					{
						$linkUrl = $cacheProducts[$id_product]->getLink();
						foreach ($products_per_group[$group_per_product[$id_product]] as $id_product2)
						{
							if (!isset($productsAttrs[$id_product2]))
								$productsAttrs[$id_product2] = array();
						
							if (!isset($productsAttrs[$id_product2][$row["id_attribute_group"]]))
								$productsAttrs[$id_product2][$row["id_attribute_group"]] = array();

							if (!isset($productsAttrs[$id_product2][$row["id_attribute_group"]][$row["id_attribute"]]))
								$productsAttrs[$id_product2][$row["id_attribute_group"]][$row["id_attribute"]] = array(
																							"linkUrl" => $linkUrl,
																							"id_attribute" => $row["id_attribute"],
																							"name" => $row["name"],
																							"color" => $row["color"],
																						);
						}
					}
				}
			}
		}

		$results = array(
			"products" => $products,
			"productsDesc" => $productsDesc,
			"productsAttrs" => $productsAttrs,
		);

		$this->RelatedProductsInfoCache[$products_id[0]] = $results;

		return $results;
	}
	
	//
	// Charge les produits connexes
	//
	public function hookActionPolarisLinkLoadRelatedProducts($params)
	{
		$single_id_product = 0;
		
		if (!isset($params["multiple_mode"]) || $params["multiple_mode"] == 0)
		{
			if (isset($params["id_product"]))
			{
				// Mode paramétré
				$single_id_product = intval($params["id_product"]);
			}
			else 
			{
				// Mode détection automatique
				$tplProduct = $params["smarty"]->getTemplateVars("product");
				if ($tplProduct != null)
				{
					if (is_array($tplProduct))
					{
						if (isset($tplProduct["id_product"]))
							$single_id_product = intval($tplProduct["id_product"]);
					}
					else
						$single_id_product = intval($tplProduct->id);
				}
			}
		}

		$products = array();
		// Deux versions, selon la version de Prestashop		
		if (version_compare(_PS_VERSION_, 1.7) < 0)
		{
			// Version 1.6
			if ($single_id_product > 0)
			{
				$products[$single_id_product] = array();
			}
			else
			{		
				// Toujours là ?
				if (isset($params["id_products"]))
				{
					// Mode paramétré
					if (is_array($params["id_products"]))
					{
						foreach ($params["id_products"] as $id)
							$product[$id] = array();
					}
					else
					{
						$products[$params["id_products"]] = array();
					}
				}
				else
				{
					// Mode automatique
					$tplProducts = $params["smarty"]->getTemplateVars("products");
					if ($tplProducts != null)
					{
						foreach ($tplProducts as $tplProduct)
							$products[$tplProduct["id_product"]] = array();
					}
				}
			}
					
			$results = $this->loadRelatedProductsInfo(array_keys($products));

			$params["smarty"]->assign("relatedProductsAttributes", 
												array(
													'img_col_dir' => _THEME_COL_DIR_,
													'col_img_dir' => _PS_COL_IMG_DIR_,
													'products' => $results["productsAttrs"],
													'details' => $results["productsDesc"],
													));

			if ($single_id_product > 0)
				$params["smarty"]->assign("relatedProducts", $results["products"][$single_id_product]);
			else
				$params["smarty"]->assign("relatedProducts", $results["products"]);
		}
		else
		{
			// Version 1.7			
			if ($single_id_product > 0)
			{
				$products[$single_id_product] = array();
			}
			else
			{		
				// Toujours là ?
				if (isset($params["id_products"]))
				{
					// Mode paramétré
					if (is_array($params["id_products"]))
					{
						foreach ($params["id_products"] as $id)
							$product[$id] = array();
					}
					else
					{
						$products[$params["id_products"]] = array();
					}
				}
				else
				{
					// Mode automatique
					$tplProducts = $params["smarty"]->getTemplateVars("listing");
					$tplProducts = $tplProducts["products"];
					if ($tplProducts != null)
					{
						foreach ($tplProducts as $tplProduct)
							$products[$tplProduct["id_product"]] = array();
					}
				}
			}

			$results = $this->loadRelatedProductsInfo(array_keys($products));

			// Conversion produits
			$results["products"] = $this->convertToPS17Presenter($results["products"]);

			$params["smarty"]->assign("relatedProductsAttributes", 
												array(
													'img_col_dir' => _THEME_COL_DIR_,
													'col_img_dir' => _PS_COL_IMG_DIR_,
													'products' => $results["productsAttrs"],
													'details' => $results["productsDesc"],
													));
					
			if ($single_id_product > 0)
				$params["smarty"]->assign("relatedProducts", $results["products"][$single_id_product]);
			else
				$params["smarty"]->assign("relatedProducts", $results["products"]);
		}
	}


	public function ps17PresentProduct($factory, $productObj)
	{
		if (empty($productObj->id))
			return null;
			
		$context = $this->context;
		$productSettings = $factory->getPresentationSettings();
		$productController = $context->controller;

		// Hook displayProductExtraContent
		$extraContentFinder = new \PrestaShop\PrestaShop\Core\Product\ProductExtraContentFinder();

		$product = $productController->objectPresenter->present($productObj);
		$product['id_product'] = (int) $productObj->id;
		$product['out_of_stock'] = (int) $productObj->out_of_stock;
		$product['new'] = (int) $productObj->new;
		//$product['id_product_attribute'] = $this->getIdProductAttributeByRequestOrGroup();
		$product['id_product_attribute'] = Product::getDefaultAttribute($productObj->id);
		//$product['minimal_quantity'] = $this->getProductMinimalQuantity($product);
		$product['minimal_quantity'] = 0;
		//$product['quantity_wanted'] = $this->getRequiredQuantity($product);
		$product['quantity_wanted'] = 0;
		$product['extraContent'] = $extraContentFinder->addParams(array('product' => $productObj))->present();
		$product['ecotax'] = Tools::convertPrice((float) $product['ecotax'], $context->currency, true, $context);

		$product_full = array_merge($product, (array) Product::getProductProperties($context->language->id, $product, $context));
		$product_full['customizations'] = array('fields' => array(),);
		$product_full['id_customization'] = 0;
		$product_full['is_customizable'] = false;

		$product_full['show_quantities'] = (bool) (
																								Configuration::get('PS_DISPLAY_QTIES')
																								&& Configuration::get('PS_STOCK_MANAGEMENT')
																								&& $productObj->quantity > 0
																								&& $productObj->available_for_order
																								&& !Configuration::isCatalogMode()
																								);
		$product_full['quantity_label'] = ($productObj->quantity > 1) ? $this->trans('Items', array(), 'Shop.Theme.Catalog') : $this->trans('Item', array(), 'Shop.Theme.Catalog');
		//$product_full['quantity_discounts'] = $this->quantity_discounts;
		$product_full['quantity_discounts'] = 0;

		if (isset($product_full['unit_price_ratio']) && $product_full['unit_price_ratio'] > 0) 
		{
			$unitPrice = ($productSettings->include_taxes) ? $product_full['price'] : $product_full['price_tax_exc'];
			$product_full['unit_price'] = $unitPrice / $product_full['unit_price_ratio'];
		}

		$group_reduction = GroupReduction::getValueForProduct($productObj->id, (int) Group::getCurrent()->id);
		if ($group_reduction === false) 
			$group_reduction = Group::getReduction((int) $context->cookie->id_customer) / 100;

		$product_full['customer_group_discount'] = $group_reduction;

		$presenter = $factory->getPresenter();

		return $presenter->present( $productSettings, $product_full, $this->context->language );		
	}

	public function convertToPS17Presenter($data)
	{		
		$factory = new ProductPresenterFactory($this->context, new TaxConfiguration());

		$results = array();		
		foreach ($data as $k => $products)
		{
			foreach ($products as $product)
			{
				if ($presentedProduct = $this->ps17PresentProduct($factory, $product))
					$results[$k][] = $presentedProduct;
			}
		}

		return $results;
	}
}
